@@ -82,7 +82,6 @@ thread_sync(void *arg)
       if (ret == 1) {
          drm_dbg("fence signaled: %p (%" PRIu64 ")", (void*)fence, fence->fence_id);
          timeline->fence_retire(timeline->vctx, timeline->ring_idx, fence->fence_id);
-         write_eventfd(timeline->eventfd, 1);
          drm_fence_destroy(fence);
       } else if (ret != 0) {
          drm_err("poll failed: %s", strerror(errno));
@@ -95,12 +94,11 @@ thread_sync(void *arg)
 
 void
 drm_timeline_init(struct drm_timeline *timeline, struct virgl_context *vctx,
-                  const char *name, int eventfd, int ring_idx,
+                  const char *name, int ring_idx,
                   virgl_context_fence_retire fence_retire)
 {
    timeline->vctx = vctx;
    timeline->name = name;
-   timeline->eventfd = eventfd;
    timeline->ring_idx = ring_idx;
    timeline->fence_retire = fence_retire;
 
@@ -160,6 +158,9 @@ drm_timeline_submit_fence(struct drm_timeline *timeline, uint32_t flags,
    cnd_signal(&timeline->fence_cond);
    mtx_unlock(&timeline->fence_mutex);
 
+   close(timeline->last_fence_fd);
+   timeline->last_fence_fd = -1;
+
    return 0;
 }
 
