@@ -9,6 +9,7 @@
 #include <inttypes.h>
 #include <stddef.h>
 #include <stdint.h>
+#include <string.h>
 #include <unistd.h>
 
 #include <xf86drm.h>
@@ -25,6 +26,10 @@
 #  include "amdgpu/amdgpu_renderer.h"
 #endif
 
+#ifdef ENABLE_DRM_ASAHI
+#  include "asahi/asahi_renderer.h"
+#endif
+
 static struct virgl_renderer_capset_drm capset;
 
 static const struct backend {
@@ -49,6 +54,14 @@ static const struct backend {
       .create = amdgpu_renderer_create,
    },
 #endif
+#ifdef ENABLE_DRM_ASAHI
+   {
+      .context_type = VIRTGPU_DRM_CONTEXT_ASAHI,
+      .name = "asahi",
+      .probe = asahi_renderer_probe,
+      .create = asahi_renderer_create,
+   },
+#endif
 };
 
 int
@@ -140,6 +153,14 @@ drm_renderer_create(size_t debug_len, const char *debug_name)
       if (fd < 0)
          return NULL;
 
+      if (debug_len && debug_name) {
+         struct drm_set_client_name n = {
+            .name_len = debug_len,
+            .name = (uint64_t) debug_name
+         };
+         drmIoctl(fd, DRM_IOCTL_SET_CLIENT_NAME, &n);
+      }
+
       return b->create(fd, debug_len, debug_name);
    }
 
