@@ -46,13 +46,13 @@ struct dump_ctx
 {
    struct tgsi_iterate_context iter;
 
-   boolean dump_float_as_hex;
+   bool dump_float_as_hex;
 
-   uint instno;
-   uint immno;
+   unsigned instno;
+   unsigned immno;
    int indent;
    
-   uint indentation;
+   unsigned indentation;
    tgsi_dump_callback_type logger;
    void *user_data;
 
@@ -84,9 +84,9 @@ log_to_file(const char *fmt, va_list ap, void* user_data)
 static void
 dump_enum(
    struct dump_ctx *ctx,
-   uint e,
+   unsigned e,
    const char **enums,
-   uint enum_count )
+   unsigned enum_count )
 {
    if (e >= enum_count)
       ctx->dump_printf( ctx, "%u", e );
@@ -232,7 +232,7 @@ _dump_register_dst(
 static void
 _dump_writemask(
    struct dump_ctx *ctx,
-   uint writemask )
+   unsigned writemask )
 {
    if (writemask != TGSI_WRITEMASK_XYZW) {
       CHR( '.' );
@@ -306,13 +306,13 @@ dump_imm_data(struct tgsi_iterate_context *iter,
    return true;
 }
 
-static boolean
+static bool
 iter_declaration(
    struct tgsi_iterate_context *iter,
    struct tgsi_full_declaration *decl )
 {
    struct dump_ctx *ctx = (struct dump_ctx *)iter;
-   boolean patch = decl->Semantic.Name == TGSI_SEMANTIC_PATCH ||
+   bool patch = decl->Semantic.Name == TGSI_SEMANTIC_PATCH ||
       decl->Semantic.Name == TGSI_SEMANTIC_TESSINNER ||
       decl->Semantic.Name == TGSI_SEMANTIC_TESSOUTER ||
       decl->Semantic.Name == TGSI_SEMANTIC_PRIMID;
@@ -459,7 +459,7 @@ iter_declaration(
 
    EOL();
 
-   return TRUE;
+   return true;
 }
 
 void
@@ -475,7 +475,7 @@ tgsi_dump_declaration(
    iter_declaration( &ctx.iter, (struct tgsi_full_declaration *)decl );
 }
 
-static boolean
+static bool
 iter_property(
    struct tgsi_iterate_context *iter,
    struct tgsi_full_property *prop )
@@ -510,7 +510,7 @@ iter_property(
    }
    EOL();
 
-   return TRUE;
+   return true;
 }
 
 void tgsi_dump_property(
@@ -525,7 +525,7 @@ void tgsi_dump_property(
    iter_property( &ctx.iter, (struct tgsi_full_property *)prop );
 }
 
-static boolean
+static bool
 iter_immediate(
    struct tgsi_iterate_context *iter,
    struct tgsi_full_immediate *imm )
@@ -543,7 +543,7 @@ iter_immediate(
 
    EOL();
 
-   return TRUE;
+   return true;
 }
 
 void
@@ -559,16 +559,16 @@ tgsi_dump_immediate(
    iter_immediate( &ctx.iter, (struct tgsi_full_immediate *)imm );
 }
 
-static boolean
+static bool
 iter_instruction(
    struct tgsi_iterate_context *iter,
    struct tgsi_full_instruction *inst )
 {
    struct dump_ctx *ctx = (struct dump_ctx *) iter;
-   uint instno = ctx->instno++;
+   unsigned instno = ctx->instno++;
    const struct tgsi_opcode_info *info = tgsi_get_opcode_info( inst->Instruction.Opcode );
-   uint i;
-   boolean first_reg = TRUE;
+   unsigned i;
+   bool first_reg = true;
 
    INSTID( instno );
    TXT( ": " );
@@ -594,7 +594,7 @@ iter_instruction(
       _dump_register_dst( ctx, dst );
       _dump_writemask( ctx, dst->Register.WriteMask );
 
-      first_reg = FALSE;
+      first_reg = false;
    }
 
    for (i = 0; i < inst->Instruction.NumSrcRegs; i++) {
@@ -625,7 +625,7 @@ iter_instruction(
       if (src->Register.Absolute)
          CHR( '|' );
 
-      first_reg = FALSE;
+      first_reg = false;
    }
 
    if (inst->Instruction.Texture) {
@@ -690,13 +690,13 @@ iter_instruction(
 
    EOL();
 
-   return TRUE;
+   return true;
 }
 
 void
 tgsi_dump_instruction(
    const struct tgsi_full_instruction *inst,
-   uint instno )
+   unsigned instno )
 {
    struct dump_ctx ctx;
 
@@ -711,21 +711,21 @@ tgsi_dump_instruction(
    iter_instruction( &ctx.iter, (struct tgsi_full_instruction *)inst );
 }
 
-static boolean
+static bool
 prolog(
    struct tgsi_iterate_context *iter )
 {
    struct dump_ctx *ctx = (struct dump_ctx *) iter;
    ENM( iter->processor.Processor, tgsi_processor_type_names );
    EOL();
-   return TRUE;
+   return true;
 }
 
 
 void
 tgsi_dump_with_logger(
    const struct tgsi_token *tokens,
-   uint flags,
+   unsigned flags,
    tgsi_dump_callback_type logger,
    void *user_data)
 {
@@ -747,15 +747,15 @@ tgsi_dump_with_logger(
    ctx.user_data = user_data;
 
    if (flags & TGSI_DUMP_FLOAT_AS_HEX)
-      ctx.dump_float_as_hex = TRUE;
+      ctx.dump_float_as_hex = true;
    else
-      ctx.dump_float_as_hex = FALSE;
+      ctx.dump_float_as_hex = false;
 
    tgsi_iterate_shader( tokens, &ctx.iter );
 }
 
 void
-tgsi_dump_to_file(const struct tgsi_token *tokens, uint flags, FILE *file)
+tgsi_dump_to_file(const struct tgsi_token *tokens, unsigned flags, FILE *file)
 {
    struct dump_ctx ctx;
 
@@ -775,15 +775,15 @@ tgsi_dump_to_file(const struct tgsi_token *tokens, uint flags, FILE *file)
    ctx.user_data = (void*)file;
 
    if (flags & TGSI_DUMP_FLOAT_AS_HEX)
-      ctx.dump_float_as_hex = TRUE;
+      ctx.dump_float_as_hex = true;
    else
-      ctx.dump_float_as_hex = FALSE;
+      ctx.dump_float_as_hex = false;
 
    tgsi_iterate_shader( tokens, &ctx.iter );
 }
 
 void
-tgsi_dump(const struct tgsi_token *tokens, uint flags)
+tgsi_dump(const struct tgsi_token *tokens, unsigned flags)
 {
    tgsi_dump_to_file(tokens, flags, NULL);
 }
@@ -824,7 +824,7 @@ str_dump_ctx_printf(struct dump_ctx *ctx, const char *format, ...)
 bool
 tgsi_dump_str(
    const struct tgsi_token *tokens,
-   uint flags,
+   unsigned flags,
    char *str,
    size_t size)
 {
@@ -852,11 +852,11 @@ tgsi_dump_str(
    ctx.nospace = false;
 
    if (flags & TGSI_DUMP_FLOAT_AS_HEX)
-      ctx.base.dump_float_as_hex = TRUE;
+      ctx.base.dump_float_as_hex = true;
    else
-      ctx.base.dump_float_as_hex = FALSE;
+      ctx.base.dump_float_as_hex = false;
 
-   boolean success = tgsi_iterate_shader( tokens, &ctx.base.iter );
+   bool success = tgsi_iterate_shader( tokens, &ctx.base.iter );
 
    return !ctx.nospace && success;
 }
@@ -864,7 +864,7 @@ tgsi_dump_str(
 void
 tgsi_dump_instruction_str(
    const struct tgsi_full_instruction *inst,
-   uint instno,
+   unsigned instno,
    char *str,
    size_t size)
 {
