@@ -82,13 +82,13 @@ struct tgsi_opcode_info
 };
 
 const struct tgsi_opcode_info *
-tgsi_get_opcode_info( uint opcode );
+tgsi_get_opcode_info( unsigned opcode );
 
 const char *
-tgsi_get_opcode_name( uint opcode );
+tgsi_get_opcode_name( unsigned opcode );
 
 const char *
-tgsi_get_processor_name( uint processor );
+tgsi_get_processor_name( unsigned processor );
 
 enum tgsi_opcode_type {
    TGSI_TYPE_UNTYPED, /* for MOV */
@@ -102,10 +102,10 @@ enum tgsi_opcode_type {
 };
 
 enum tgsi_opcode_type
-tgsi_opcode_infer_src_type( uint opcode );
+tgsi_opcode_infer_src_type( unsigned opcode );
 
 enum tgsi_opcode_type
-tgsi_opcode_infer_dst_type( uint opcode );
+tgsi_opcode_infer_dst_type( unsigned opcode );
 
 #if defined __cplusplus
 }
