@@ -30,7 +30,7 @@
 #define U_FORMAT_H
 
 
-#include "pipe/p_format.h"
+#include "util/u_formats.h"
 #include "pipe/p_defines.h"
 #include "util/u_debug.h"
 
@@ -107,6 +107,9 @@ struct util_format_block
    /** Block height in pixels */
    unsigned height;
 
+   /** Block depth in pixels */
+   unsigned depth;
+
    /** Block size in bits */
    unsigned bits;
 };
@@ -121,18 +124,6 @@ enum util_format_type {
 };
 
 
-enum util_format_swizzle {
-   UTIL_FORMAT_SWIZZLE_X = 0,
-   UTIL_FORMAT_SWIZZLE_Y = 1,
-   UTIL_FORMAT_SWIZZLE_Z = 2,
-   UTIL_FORMAT_SWIZZLE_W = 3,
-   UTIL_FORMAT_SWIZZLE_0 = 4,
-   UTIL_FORMAT_SWIZZLE_1 = 5,
-   UTIL_FORMAT_SWIZZLE_NONE = 6,
-   UTIL_FORMAT_SWIZZLE_MAX = 7  /**< Number of enums counter (must be last) */
-};
-
-
 enum util_format_colorspace {
    UTIL_FORMAT_COLORSPACE_RGB = 0,
    UTIL_FORMAT_COLORSPACE_SRGB = 1,
@@ -193,6 +184,16 @@ struct util_format_description
     */
    unsigned is_mixed:1;
 
+   /**
+    * Whether the format contains UNORM channels
+    */
+   unsigned is_unorm:1;
+
+   /**
+    * Whether the format contains SNORM channels
+    */
+   unsigned is_snorm:1;
+
    /**
     * Input channel description, in the order XYZW.
     *
@@ -237,6 +238,16 @@ struct util_format_description
     * Colorspace transformation.
     */
    enum util_format_colorspace colorspace;
+
+   /**
+    * For sRGB formats, equivalent linear format; for linear formats,
+    * equivalent sRGB format
+    */
+   union {
+      enum pipe_format srgb_equivalent;
+      enum pipe_format linear_equivalent;
+   };
+
 };
 
 
@@ -244,7 +255,7 @@ extern const struct util_format_description
 util_format_description_table[];
 
 
-const struct util_format_description *
+ATTRIBUTE_RETURNS_NONNULL const struct util_format_description *
 util_format_description(enum pipe_format format);
 
 
@@ -252,24 +263,26 @@ util_format_description(enum pipe_format format);
  * Format query functions.
  */
 
-static inline const char *
+ATTRIBUTE_RETURNS_NONNULL static inline const char *
 util_format_name(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   if (!desc) {
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT || !desc->name)) {
       return "PIPE_FORMAT_???";
    }
 
    return desc->name;
 }
 
-static inline const char *
+ATTRIBUTE_RETURNS_NONNULL static inline const char *
 util_format_short_name(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   if (!desc) {
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT || !desc->short_name)) {
       return "???";
    }
 
@@ -279,25 +292,27 @@ util_format_short_name(enum pipe_format format)
 /**
  * Whether this format is plain, see UTIL_FORMAT_LAYOUT_PLAIN for more info.
  */
-static inline boolean
+static inline bool
 util_format_is_plain(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   if (!format) {
-      return FALSE;
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
+      return false;
    }
 
-   return desc->layout == UTIL_FORMAT_LAYOUT_PLAIN ? TRUE : FALSE;
+   return desc->layout == UTIL_FORMAT_LAYOUT_PLAIN;
 }
 
-static inline boolean 
+static inline bool
 util_format_is_compressed(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   if (!desc) {
-      return FALSE;
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
+      return false;
    }
 
    switch (desc->layout) {
@@ -309,53 +324,61 @@ util_format_is_compressed(enum pipe_format format)
    case UTIL_FORMAT_LAYOUT_ATC:
    case UTIL_FORMAT_LAYOUT_FXT1:
       /* XXX add other formats in the future */
-      return TRUE;
+      return true;
    default:
-      return FALSE;
+      return false;
    }
 }
 
-static inline boolean 
+static inline bool
 util_format_is_srgb(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
+
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
+      return false;
+   }
+
    return desc->colorspace == UTIL_FORMAT_COLORSPACE_SRGB;
 }
 
-static inline boolean
+static inline bool
 util_format_has_depth(const struct util_format_description *desc)
 {
    return desc->colorspace == UTIL_FORMAT_COLORSPACE_ZS &&
-          desc->swizzle[0] != UTIL_FORMAT_SWIZZLE_NONE;
+          desc->swizzle[0] != PIPE_SWIZZLE_NONE;
 }
 
-static inline boolean
+static inline bool
 util_format_has_stencil(const struct util_format_description *desc)
 {
    return desc->colorspace == UTIL_FORMAT_COLORSPACE_ZS &&
-          desc->swizzle[1] != UTIL_FORMAT_SWIZZLE_NONE;
+          desc->swizzle[1] != PIPE_SWIZZLE_NONE;
 }
 
-static inline boolean
+static inline bool
 util_format_is_depth_or_stencil(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   if (!desc) {
-      return FALSE;
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
+      return false;
    }
 
    return util_format_has_depth(desc) ||
           util_format_has_stencil(desc);
 }
 
-static inline boolean
+static inline bool
 util_format_is_depth_and_stencil(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   if (!desc) {
-      return FALSE;
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
+      return false;
    }
 
    return util_format_has_depth(desc) &&
@@ -371,7 +394,7 @@ util_get_depth_format_type(const struct util_format_description *desc)
 {
    unsigned depth_channel = desc->swizzle[0];
    if (desc->colorspace == UTIL_FORMAT_COLORSPACE_ZS &&
-       depth_channel != UTIL_FORMAT_SWIZZLE_NONE) {
+       depth_channel != PIPE_SWIZZLE_NONE) {
       return desc->channel[depth_channel].type;
    } else {
       return UTIL_FORMAT_TYPE_VOID;
@@ -399,8 +422,10 @@ util_format_get_mask(enum pipe_format format)
    const struct util_format_description *desc =
       util_format_description(format);
 
-   if (!desc)
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
       return 0;
+   }
 
    if (util_format_has_depth(desc)) {
       if (util_format_has_stencil(desc)) {
@@ -417,33 +442,33 @@ util_format_get_mask(enum pipe_format format)
    }
 }
 
-boolean
+bool
 util_format_has_alpha(enum pipe_format format);
 
 
-boolean
+bool
 util_format_is_luminance(enum pipe_format format);
 
-boolean
+bool
 util_format_is_alpha(enum pipe_format format);
 
-boolean
+bool
 util_format_is_luminance_alpha(enum pipe_format format);
 
 
-boolean
+bool
 util_format_is_intensity(enum pipe_format format);
 
-boolean
+bool
 util_format_is_pure_integer(enum pipe_format format);
 
-boolean
+bool
 util_format_is_pure_sint(enum pipe_format format);
 
-boolean
+bool
 util_format_is_pure_uint(enum pipe_format format);
 
-boolean
+bool
 util_format_is_snorm(enum pipe_format format);
 
 /**
@@ -451,7 +476,7 @@ util_format_is_snorm(enum pipe_format format);
  * a simple memcpy.  For example, blitting from RGBA to RGBx is OK, but not
  * the reverse.
  */
-boolean
+bool
 util_is_format_compatible(const struct util_format_description *src_desc,
                           const struct util_format_description *dst_desc);
 
@@ -462,7 +487,7 @@ util_is_format_compatible(const struct util_format_description *src_desc,
  *
  *   PIPE_FORMAT_?8?8?8?8_UNORM
  */
-static inline boolean
+static inline bool
 util_format_is_rgba8_variant(const struct util_format_description *desc)
 {
    unsigned chan;
@@ -470,32 +495,33 @@ util_format_is_rgba8_variant(const struct util_format_description *desc)
    if(desc->block.width != 1 ||
       desc->block.height != 1 ||
       desc->block.bits != 32)
-      return FALSE;
+      return false;
 
    for(chan = 0; chan < 4; ++chan) {
       if(desc->channel[chan].type != UTIL_FORMAT_TYPE_UNSIGNED &&
          desc->channel[chan].type != UTIL_FORMAT_TYPE_VOID)
-         return FALSE;
+         return false;
       if(desc->channel[chan].type == UTIL_FORMAT_TYPE_UNSIGNED &&
          !desc->channel[chan].normalized)
-         return FALSE;
+         return false;
       if(desc->channel[chan].size != 8)
-         return FALSE;
+         return false;
    }
 
-   return TRUE;
+   return true;
 }
 
 
 /**
  * Return total bits needed for the pixel format per block.
  */
-static inline uint
+static inline unsigned
 util_format_get_blocksizebits(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   if (!desc) {
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
       return 0;
    }
 
@@ -505,11 +531,11 @@ util_format_get_blocksizebits(enum pipe_format format)
 /**
  * Return bytes per block (not pixel) for the given format.
  */
-static inline uint
+static inline unsigned
 util_format_get_blocksize(enum pipe_format format)
 {
-   uint bits = util_format_get_blocksizebits(format);
-   uint bytes = bits / 8;
+   unsigned bits = util_format_get_blocksizebits(format);
+   unsigned bytes = bits / 8;
 
    assert(bits % 8 == 0);
    assert(bytes > 0);
@@ -520,30 +546,45 @@ util_format_get_blocksize(enum pipe_format format)
    return bytes;
 }
 
-static inline uint
+static inline unsigned
 util_format_get_blockwidth(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   if (!desc) {
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
       return 1;
    }
 
    return desc->block.width;
 }
 
-static inline uint
+static inline unsigned
 util_format_get_blockheight(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
 
-   if (!desc) {
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
       return 1;
    }
 
    return desc->block.height;
 }
 
+static inline unsigned
+util_format_get_blockdepth(enum pipe_format format)
+{
+   const struct util_format_description *desc = util_format_description(format);
+
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
+      return 1;
+   }
+
+   return desc->block.depth;
+}
+
 static inline unsigned
 util_format_get_nblocksx(enum pipe_format format,
                          unsigned x)
@@ -560,11 +601,20 @@ util_format_get_nblocksy(enum pipe_format format,
    return (y + blockheight - 1) / blockheight;
 }
 
+static inline unsigned
+util_format_get_nblocksz(enum pipe_format format,
+                         unsigned z)
+{
+   unsigned blockdepth = util_format_get_blockdepth(format);
+   return (z + blockdepth - 1) / blockdepth;
+}
+
 static inline unsigned
 util_format_get_nblocks(enum pipe_format format,
                         unsigned width,
                         unsigned height)
 {
+   assert(util_format_get_blockdepth(format) == 1);
    return util_format_get_nblocksx(format, width) * util_format_get_nblocksy(format, height);
 }
 
@@ -583,10 +633,10 @@ util_format_get_2d_size(enum pipe_format format,
    return util_format_get_nblocksy(format, height) * stride;
 }
 
-static inline uint
+static inline unsigned
 util_format_get_component_bits(enum pipe_format format,
                                enum util_format_colorspace colorspace,
-                               uint component)
+                               unsigned component)
 {
    const struct util_format_description *desc = util_format_description(format);
    enum util_format_colorspace desc_colorspace;
@@ -613,13 +663,13 @@ util_format_get_component_bits(enum pipe_format format,
    }
 
    switch (desc->swizzle[component]) {
-   case UTIL_FORMAT_SWIZZLE_X:
+   case PIPE_SWIZZLE_X:
       return desc->channel[0].size;
-   case UTIL_FORMAT_SWIZZLE_Y:
+   case PIPE_SWIZZLE_Y:
       return desc->channel[1].size;
-   case UTIL_FORMAT_SWIZZLE_Z:
+   case PIPE_SWIZZLE_Z:
       return desc->channel[2].size;
-   case UTIL_FORMAT_SWIZZLE_W:
+   case PIPE_SWIZZLE_W:
       return desc->channel[3].size;
    default:
       return 0;
@@ -636,44 +686,12 @@ util_format_srgb(enum pipe_format format)
    if (util_format_is_srgb(format))
       return format;
 
-   switch (format) {
-   case PIPE_FORMAT_L8_UNORM:
-      return PIPE_FORMAT_L8_SRGB;
-   case PIPE_FORMAT_L8A8_UNORM:
-      return PIPE_FORMAT_L8A8_SRGB;
-   case PIPE_FORMAT_R8G8B8_UNORM:
-      return PIPE_FORMAT_R8G8B8_SRGB;
-   case PIPE_FORMAT_A8B8G8R8_UNORM:
-      return PIPE_FORMAT_A8B8G8R8_SRGB;
-   case PIPE_FORMAT_X8B8G8R8_UNORM:
-      return PIPE_FORMAT_X8B8G8R8_SRGB;
-   case PIPE_FORMAT_B8G8R8A8_UNORM:
-      return PIPE_FORMAT_B8G8R8A8_SRGB;
-   case PIPE_FORMAT_B8G8R8X8_UNORM:
-      return PIPE_FORMAT_B8G8R8X8_SRGB;
-   case PIPE_FORMAT_A8R8G8B8_UNORM:
-      return PIPE_FORMAT_A8R8G8B8_SRGB;
-   case PIPE_FORMAT_X8R8G8B8_UNORM:
-      return PIPE_FORMAT_X8R8G8B8_SRGB;
-   case PIPE_FORMAT_R8G8B8A8_UNORM:
-      return PIPE_FORMAT_R8G8B8A8_SRGB;
-   case PIPE_FORMAT_R8G8B8X8_UNORM:
-      return PIPE_FORMAT_R8G8B8X8_SRGB;
-   case PIPE_FORMAT_DXT1_RGB:
-      return PIPE_FORMAT_DXT1_SRGB;
-   case PIPE_FORMAT_DXT1_RGBA:
-      return PIPE_FORMAT_DXT1_SRGBA;
-   case PIPE_FORMAT_DXT3_RGBA:
-      return PIPE_FORMAT_DXT3_SRGBA;
-   case PIPE_FORMAT_DXT5_RGBA:
-      return PIPE_FORMAT_DXT5_SRGBA;
-   case PIPE_FORMAT_B5G6R5_UNORM:
-      return PIPE_FORMAT_B5G6R5_SRGB;
-   case PIPE_FORMAT_BPTC_RGBA_UNORM:
-      return PIPE_FORMAT_BPTC_SRGBA;
-   default:
-      return PIPE_FORMAT_NONE;
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
+      return format;
    }
+
+   return util_format_description(format)->srgb_equivalent;
 }
 
 /**
@@ -683,44 +701,15 @@ util_format_srgb(enum pipe_format format)
 static inline enum pipe_format
 util_format_linear(enum pipe_format format)
 {
-   switch (format) {
-   case PIPE_FORMAT_L8_SRGB:
-      return PIPE_FORMAT_L8_UNORM;
-   case PIPE_FORMAT_L8A8_SRGB:
-      return PIPE_FORMAT_L8A8_UNORM;
-   case PIPE_FORMAT_R8G8B8_SRGB:
-      return PIPE_FORMAT_R8G8B8_UNORM;
-   case PIPE_FORMAT_A8B8G8R8_SRGB:
-      return PIPE_FORMAT_A8B8G8R8_UNORM;
-   case PIPE_FORMAT_X8B8G8R8_SRGB:
-      return PIPE_FORMAT_X8B8G8R8_UNORM;
-   case PIPE_FORMAT_B8G8R8A8_SRGB:
-      return PIPE_FORMAT_B8G8R8A8_UNORM;
-   case PIPE_FORMAT_B8G8R8X8_SRGB:
-      return PIPE_FORMAT_B8G8R8X8_UNORM;
-   case PIPE_FORMAT_A8R8G8B8_SRGB:
-      return PIPE_FORMAT_A8R8G8B8_UNORM;
-   case PIPE_FORMAT_X8R8G8B8_SRGB:
-      return PIPE_FORMAT_X8R8G8B8_UNORM;
-   case PIPE_FORMAT_R8G8B8A8_SRGB:
-      return PIPE_FORMAT_R8G8B8A8_UNORM;
-   case PIPE_FORMAT_R8G8B8X8_SRGB:
-      return PIPE_FORMAT_R8G8B8X8_UNORM;
-   case PIPE_FORMAT_DXT1_SRGB:
-      return PIPE_FORMAT_DXT1_RGB;
-   case PIPE_FORMAT_DXT1_SRGBA:
-      return PIPE_FORMAT_DXT1_RGBA;
-   case PIPE_FORMAT_DXT3_SRGBA:
-      return PIPE_FORMAT_DXT3_RGBA;
-   case PIPE_FORMAT_DXT5_SRGBA:
-      return PIPE_FORMAT_DXT5_RGBA;
-   case PIPE_FORMAT_B5G6R5_SRGB:
-      return PIPE_FORMAT_B5G6R5_UNORM;
-   case PIPE_FORMAT_BPTC_SRGBA:
-      return PIPE_FORMAT_BPTC_RGBA_UNORM;
-   default:
+   if (!util_format_is_srgb(format))
+      return format;
+
+   assert(format < PIPE_FORMAT_COUNT);
+   if (unlikely(format >= PIPE_FORMAT_COUNT)) {
       return format;
    }
+
+   return util_format_description(format)->linear_equivalent;
 }
 
 /**
@@ -875,6 +864,8 @@ static inline unsigned
 util_format_get_nr_components(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
+   assert(desc->nr_channels <= 4);
+
    return desc->nr_channels;
 }
 
@@ -902,7 +893,7 @@ util_format_get_first_non_void_channel(enum pipe_format format)
  * Generic format conversion;
  */
 
-boolean
+bool
 util_format_fits_8unorm(const struct util_format_description *format_desc);
 
 /*
@@ -925,7 +916,7 @@ void util_format_compose_swizzles(const unsigned char swz1[4],
 void util_format_apply_color_swizzle(union pipe_color_union *dst,
                                      const union pipe_color_union *src,
                                      const unsigned char swz[4],
-                                     const boolean is_integer);
+                                     const bool is_integer);
 
 void util_format_swizzle_4f(float *dst, const float *src,
                             const unsigned char swz[4]);
