@@ -208,6 +208,18 @@ enum tgsi_semantic {
    TGSI_SEMANTIC_TESSINNER,   /**< inner tessellation levels */
    TGSI_SEMANTIC_VERTICESIN,  /**< number of input vertices */
    TGSI_SEMANTIC_HELPER_INVOCATION,  /**< current invocation is helper */
+   TGSI_SEMANTIC_BASEINSTANCE,
+   TGSI_SEMANTIC_DRAWID,
+   TGSI_SEMANTIC_WORK_DIM,    /**< opencl get_work_dim value */
+   TGSI_SEMANTIC_SUBGROUP_SIZE,
+   TGSI_SEMANTIC_SUBGROUP_INVOCATION,
+   TGSI_SEMANTIC_SUBGROUP_EQ_MASK,
+   TGSI_SEMANTIC_SUBGROUP_GE_MASK,
+   TGSI_SEMANTIC_SUBGROUP_GT_MASK,
+   TGSI_SEMANTIC_SUBGROUP_LE_MASK,
+   TGSI_SEMANTIC_SUBGROUP_LT_MASK,
+   TGSI_SEMANTIC_CS_USER_DATA_AMD,
+   TGSI_SEMANTIC_VIEWPORT_MASK,
    TGSI_SEMANTIC_COUNT,       /**< number of semantic values */
 };
 
@@ -356,277 +368,265 @@ struct tgsi_property_data {
  * OR REMOVE OPCODES - FILL in and REWRITE tgsi_info
  * accordingly.
  */
+
 enum tgsi_opcode {
-   TGSI_OPCODE_ARL                = 0,
-   TGSI_OPCODE_MOV                = 1,
-   TGSI_OPCODE_LIT                = 2,
-   TGSI_OPCODE_RCP                = 3,
-   TGSI_OPCODE_RSQ                = 4,
-   TGSI_OPCODE_EXP                = 5,
-   TGSI_OPCODE_LOG                = 6,
-   TGSI_OPCODE_MUL                = 7,
-   TGSI_OPCODE_ADD                = 8,
-   TGSI_OPCODE_DP3                = 9,
-   TGSI_OPCODE_DP4                = 10,
-   TGSI_OPCODE_DST                = 11,
-   TGSI_OPCODE_MIN                = 12,
-   TGSI_OPCODE_MAX                = 13,
-   TGSI_OPCODE_SLT                = 14,
-   TGSI_OPCODE_SGE                = 15,
-   TGSI_OPCODE_MAD                = 16,
-   TGSI_OPCODE_SUB                = 17,
-   TGSI_OPCODE_LRP                = 18,
-   TGSI_OPCODE_FMA                = 19,
-   TGSI_OPCODE_SQRT               = 20,
-                                /* gap */
-   TGSI_OPCODE_FRC                = 24,
-                                /* gap */
-   TGSI_OPCODE_FLR                = 26,
-   TGSI_OPCODE_ROUND              = 27,
-   TGSI_OPCODE_EX2                = 28,
-   TGSI_OPCODE_LG2                = 29,
-   TGSI_OPCODE_POW                = 30,
-   TGSI_OPCODE_XPD                = 31,
-                                /* gap */
-   TGSI_OPCODE_ABS                = 33,
-                                /* gap */
-   TGSI_OPCODE_DPH                = 35,
-   TGSI_OPCODE_COS                = 36,
-   TGSI_OPCODE_DDX                = 37,
-   TGSI_OPCODE_DDY                = 38,
-   TGSI_OPCODE_KILL               = 39 /* unconditional */,
-   TGSI_OPCODE_PK2H               = 40,
-   TGSI_OPCODE_PK2US              = 41,
-   TGSI_OPCODE_PK4B               = 42,
-   TGSI_OPCODE_PK4UB              = 43,
-                                /* gap */
-   TGSI_OPCODE_SEQ                = 45,
-                                /* gap */
-   TGSI_OPCODE_SGT                = 47,
-   TGSI_OPCODE_SIN                = 48,
-   TGSI_OPCODE_SLE                = 49,
-   TGSI_OPCODE_SNE                = 50,
-                                /* gap */
-   TGSI_OPCODE_TEX                = 52,
-   TGSI_OPCODE_TXD                = 53,
-   TGSI_OPCODE_TXP                = 54,
-   TGSI_OPCODE_UP2H               = 55,
-   TGSI_OPCODE_UP2US              = 56,
-   TGSI_OPCODE_UP4B               = 57,
-   TGSI_OPCODE_UP4UB              = 58,
-                                /* gap */
-   TGSI_OPCODE_ARR                = 61,
-                                /* gap */
-   TGSI_OPCODE_CAL                = 63,
-   TGSI_OPCODE_RET                = 64,
-   TGSI_OPCODE_SSG                = 65 /* SGN */,
-   TGSI_OPCODE_CMP                = 66,
-   TGSI_OPCODE_SCS                = 67,
-   TGSI_OPCODE_TXB                = 68,
-   TGSI_OPCODE_FBFETCH            = 69,
-   TGSI_OPCODE_DIV                = 70,
-   TGSI_OPCODE_DP2                = 71,
-   TGSI_OPCODE_TXL                = 72,
-   TGSI_OPCODE_BRK                = 73,
-   TGSI_OPCODE_IF                 = 74,
-   TGSI_OPCODE_UIF                = 75,
-                                /* gap */
-   TGSI_OPCODE_ELSE               = 77,
-   TGSI_OPCODE_ENDIF              = 78,
-   TGSI_OPCODE_DDX_FINE           = 79,
-   TGSI_OPCODE_DDY_FINE           = 80,
-                                /* gap */
-   TGSI_OPCODE_CEIL               = 83,
-   TGSI_OPCODE_I2F                = 84,
-   TGSI_OPCODE_NOT                = 85,
-   TGSI_OPCODE_TRUNC              = 86,
-   TGSI_OPCODE_SHL                = 87,
-                                /* gap */
-   TGSI_OPCODE_AND                = 89,
-   TGSI_OPCODE_OR                 = 90,
-   TGSI_OPCODE_MOD                = 91,
-   TGSI_OPCODE_XOR                = 92,
-                                /* gap */
-   TGSI_OPCODE_TXF                = 94,
-   TGSI_OPCODE_TXQ                = 95,
-   TGSI_OPCODE_CONT               = 96,
-   TGSI_OPCODE_EMIT               = 97,
-   TGSI_OPCODE_ENDPRIM            = 98,
-   TGSI_OPCODE_BGNLOOP            = 99,
-   TGSI_OPCODE_BGNSUB             = 100,
-   TGSI_OPCODE_ENDLOOP            = 101,
-   TGSI_OPCODE_ENDSUB             = 102,
-                                /* gap */
-   TGSI_OPCODE_TXQS               = 104,
-   TGSI_OPCODE_RESQ               = 105,
-                                /* gap */
-   TGSI_OPCODE_NOP                = 107,
+   TGSI_OPCODE_ARL,
+   TGSI_OPCODE_MOV,
+   TGSI_OPCODE_LIT,
+   TGSI_OPCODE_RCP,
+   TGSI_OPCODE_RSQ,
+   TGSI_OPCODE_EXP,
+   TGSI_OPCODE_LOG,
+   TGSI_OPCODE_MUL,
+   TGSI_OPCODE_ADD,
+   TGSI_OPCODE_DP3,
+   TGSI_OPCODE_DP4,
+   TGSI_OPCODE_DST,
+   TGSI_OPCODE_MIN,
+   TGSI_OPCODE_MAX,
+   TGSI_OPCODE_SLT,
+   TGSI_OPCODE_SGE,
+   TGSI_OPCODE_MAD,
+   TGSI_OPCODE_SUB,
+   TGSI_OPCODE_LRP,
+   TGSI_OPCODE_FMA,
+   TGSI_OPCODE_SQRT,
+   TGSI_OPCODE_FRC,
+   TGSI_OPCODE_FLR,
+   TGSI_OPCODE_ROUND,
+   TGSI_OPCODE_EX2,
+   TGSI_OPCODE_LG2,
+   TGSI_OPCODE_POW,
+   TGSI_OPCODE_XPD,
+   TGSI_OPCODE_ABS,
+   TGSI_OPCODE_DPH,
+   TGSI_OPCODE_COS,
+   TGSI_OPCODE_DDX,
+   TGSI_OPCODE_DDY,
+   TGSI_OPCODE_KILL /* unconditional */,
+   TGSI_OPCODE_PK2H,
+   TGSI_OPCODE_PK2US,
+   TGSI_OPCODE_PK4B,
+   TGSI_OPCODE_PK4UB,
+   TGSI_OPCODE_SEQ,
+   TGSI_OPCODE_SGT,
+   TGSI_OPCODE_SIN,
+   TGSI_OPCODE_SLE,
+   TGSI_OPCODE_SNE,
+   TGSI_OPCODE_TEX,
+   TGSI_OPCODE_TXD,
+   TGSI_OPCODE_TXP,
+   TGSI_OPCODE_UP2H,
+   TGSI_OPCODE_UP2US,
+   TGSI_OPCODE_UP4B,
+   TGSI_OPCODE_UP4UB,
+   TGSI_OPCODE_ARR,
+   TGSI_OPCODE_CAL,
+   TGSI_OPCODE_RET,
+   TGSI_OPCODE_SSG /* SGN */,
+   TGSI_OPCODE_CMP,
+   TGSI_OPCODE_SCS,
+   TGSI_OPCODE_TXB,
+   TGSI_OPCODE_FBFETCH,
+   TGSI_OPCODE_DIV,
+   TGSI_OPCODE_DP2,
+   TGSI_OPCODE_TXL,
+   TGSI_OPCODE_BRK,
+   TGSI_OPCODE_IF,
+   TGSI_OPCODE_UIF,
+   TGSI_OPCODE_ELSE,
+   TGSI_OPCODE_ENDIF,
+   TGSI_OPCODE_DDX_FINE,
+   TGSI_OPCODE_DDY_FINE,
+   TGSI_OPCODE_CEIL,
+   TGSI_OPCODE_I2F,
+   TGSI_OPCODE_NOT,
+   TGSI_OPCODE_TRUNC,
+   TGSI_OPCODE_SHL,
+   TGSI_OPCODE_AND,
+   TGSI_OPCODE_OR,
+   TGSI_OPCODE_MOD,
+   TGSI_OPCODE_XOR,
+   TGSI_OPCODE_TXF,
+   TGSI_OPCODE_TXQ,
+   TGSI_OPCODE_CONT,
+   TGSI_OPCODE_EMIT,
+   TGSI_OPCODE_ENDPRIM,
+   TGSI_OPCODE_BGNLOOP,
+   TGSI_OPCODE_BGNSUB,
+   TGSI_OPCODE_ENDLOOP,
+   TGSI_OPCODE_ENDSUB,
+   TGSI_OPCODE_TXQS,
+   TGSI_OPCODE_RESQ,
+   TGSI_OPCODE_NOP,
+   TGSI_OPCODE_FSEQ,
+   TGSI_OPCODE_FSGE,
+   TGSI_OPCODE_FSLT,
+   TGSI_OPCODE_FSNE,
 
-   TGSI_OPCODE_FSEQ               = 108,
-   TGSI_OPCODE_FSGE               = 109,
-   TGSI_OPCODE_FSLT               = 110,
-   TGSI_OPCODE_FSNE               = 111,
+   TGSI_OPCODE_MEMBAR,
 
-   TGSI_OPCODE_MEMBAR             = 112,
-                                /* gap */
-   TGSI_OPCODE_KILL_IF            = 116  /* conditional kill */,
-   TGSI_OPCODE_END                = 117  /* aka HALT */,
-   TGSI_OPCODE_DFMA               = 118,
-   TGSI_OPCODE_F2I                = 119,
-   TGSI_OPCODE_IDIV               = 120,
-   TGSI_OPCODE_IMAX               = 121,
-   TGSI_OPCODE_IMIN               = 122,
-   TGSI_OPCODE_INEG               = 123,
-   TGSI_OPCODE_ISGE               = 124,
-   TGSI_OPCODE_ISHR               = 125,
-   TGSI_OPCODE_ISLT               = 126,
-   TGSI_OPCODE_F2U                = 127,
-   TGSI_OPCODE_U2F                = 128,
-   TGSI_OPCODE_UADD               = 129,
-   TGSI_OPCODE_UDIV               = 130,
-   TGSI_OPCODE_UMAD               = 131,
-   TGSI_OPCODE_UMAX               = 132,
-   TGSI_OPCODE_UMIN               = 133,
-   TGSI_OPCODE_UMOD               = 134,
-   TGSI_OPCODE_UMUL               = 135,
-   TGSI_OPCODE_USEQ               = 136,
-   TGSI_OPCODE_USGE               = 137,
-   TGSI_OPCODE_USHR               = 138,
-   TGSI_OPCODE_USLT               = 139,
-   TGSI_OPCODE_USNE               = 140,
-   TGSI_OPCODE_SWITCH             = 141,
-   TGSI_OPCODE_CASE               = 142,
-   TGSI_OPCODE_DEFAULT            = 143,
-   TGSI_OPCODE_ENDSWITCH          = 144,
+   TGSI_OPCODE_VOTE_ANY,
+   TGSI_OPCODE_VOTE_ALL,
+   TGSI_OPCODE_VOTE_EQ,
+
+   TGSI_OPCODE_KILL_IF  /* conditional kill */,
+   TGSI_OPCODE_END  /* aka HALT */,
+   TGSI_OPCODE_DFMA,
+   TGSI_OPCODE_F2I,
+   TGSI_OPCODE_IDIV,
+   TGSI_OPCODE_IMAX,
+   TGSI_OPCODE_IMIN,
+   TGSI_OPCODE_INEG,
+   TGSI_OPCODE_ISGE,
+   TGSI_OPCODE_ISHR,
+   TGSI_OPCODE_ISLT,
+   TGSI_OPCODE_F2U,
+   TGSI_OPCODE_U2F,
+   TGSI_OPCODE_UADD,
+   TGSI_OPCODE_UDIV,
+   TGSI_OPCODE_UMAD,
+   TGSI_OPCODE_UMAX,
+   TGSI_OPCODE_UMIN,
+   TGSI_OPCODE_UMOD,
+   TGSI_OPCODE_UMUL,
+   TGSI_OPCODE_USEQ,
+   TGSI_OPCODE_USGE,
+   TGSI_OPCODE_USHR,
+   TGSI_OPCODE_USLT,
+   TGSI_OPCODE_USNE,
+   TGSI_OPCODE_SWITCH,
+   TGSI_OPCODE_CASE,
+   TGSI_OPCODE_DEFAULT,
+   TGSI_OPCODE_ENDSWITCH,
 
 /* resource related opcodes */
-   TGSI_OPCODE_SAMPLE             = 145,
-   TGSI_OPCODE_SAMPLE_I           = 146,
-   TGSI_OPCODE_SAMPLE_I_MS        = 147,
-   TGSI_OPCODE_SAMPLE_B           = 148,
-   TGSI_OPCODE_SAMPLE_C           = 149,
-   TGSI_OPCODE_SAMPLE_C_LZ        = 150,
-   TGSI_OPCODE_SAMPLE_D           = 151,
-   TGSI_OPCODE_SAMPLE_L           = 152,
-   TGSI_OPCODE_GATHER4            = 153,
-   TGSI_OPCODE_SVIEWINFO          = 154,
-   TGSI_OPCODE_SAMPLE_POS         = 155,
-   TGSI_OPCODE_SAMPLE_INFO        = 156,
+   TGSI_OPCODE_SAMPLE,
+   TGSI_OPCODE_SAMPLE_I,
+   TGSI_OPCODE_SAMPLE_I_MS,
+   TGSI_OPCODE_SAMPLE_B,
+   TGSI_OPCODE_SAMPLE_C,
+   TGSI_OPCODE_SAMPLE_C_LZ,
+   TGSI_OPCODE_SAMPLE_D,
+   TGSI_OPCODE_SAMPLE_L,
+   TGSI_OPCODE_GATHER4,
+   TGSI_OPCODE_SVIEWINFO,
+   TGSI_OPCODE_SAMPLE_POS,
+   TGSI_OPCODE_SAMPLE_INFO,
 
-   TGSI_OPCODE_UARL               = 157,
-   TGSI_OPCODE_UCMP               = 158,
-   TGSI_OPCODE_IABS               = 159,
-   TGSI_OPCODE_ISSG               = 160,
+   TGSI_OPCODE_UARL,
+   TGSI_OPCODE_UCMP,
+   TGSI_OPCODE_IABS,
+   TGSI_OPCODE_ISSG,
 
-   TGSI_OPCODE_LOAD               = 161,
-   TGSI_OPCODE_STORE              = 162,
-                                /* gap */
-   TGSI_OPCODE_BARRIER            = 166,
+   TGSI_OPCODE_LOAD,
+   TGSI_OPCODE_STORE,
+   TGSI_OPCODE_BARRIER,
 
-   TGSI_OPCODE_ATOMUADD           = 167,
-   TGSI_OPCODE_ATOMXCHG           = 168,
-   TGSI_OPCODE_ATOMCAS            = 169,
-   TGSI_OPCODE_ATOMAND            = 170,
-   TGSI_OPCODE_ATOMOR             = 171,
-   TGSI_OPCODE_ATOMXOR            = 172,
-   TGSI_OPCODE_ATOMUMIN           = 173,
-   TGSI_OPCODE_ATOMUMAX           = 174,
-   TGSI_OPCODE_ATOMIMIN           = 175,
-   TGSI_OPCODE_ATOMIMAX           = 176,
+   TGSI_OPCODE_ATOMUADD,
+   TGSI_OPCODE_ATOMXCHG,
+   TGSI_OPCODE_ATOMCAS,
+   TGSI_OPCODE_ATOMAND,
+   TGSI_OPCODE_ATOMOR,
+   TGSI_OPCODE_ATOMXOR,
+   TGSI_OPCODE_ATOMUMIN,
+   TGSI_OPCODE_ATOMUMAX,
+   TGSI_OPCODE_ATOMIMIN,
+   TGSI_OPCODE_ATOMIMAX,
 
 /* to be used for shadow cube map compares */
-   TGSI_OPCODE_TEX2               = 177,
-   TGSI_OPCODE_TXB2               = 178,
-   TGSI_OPCODE_TXL2               = 179,
+   TGSI_OPCODE_TEX2,
+   TGSI_OPCODE_TXB2,
+   TGSI_OPCODE_TXL2,
 
-   TGSI_OPCODE_IMUL_HI            = 180,
-   TGSI_OPCODE_UMUL_HI            = 181,
+   TGSI_OPCODE_IMUL_HI,
+   TGSI_OPCODE_UMUL_HI,
 
-   TGSI_OPCODE_TG4                = 182,
+   TGSI_OPCODE_TG4,
 
-   TGSI_OPCODE_LODQ               = 183,
+   TGSI_OPCODE_LODQ,
 
-   TGSI_OPCODE_IBFE               = 184,
-   TGSI_OPCODE_UBFE               = 185,
-   TGSI_OPCODE_BFI                = 186,
-   TGSI_OPCODE_BREV               = 187,
-   TGSI_OPCODE_POPC               = 188,
-   TGSI_OPCODE_LSB                = 189,
-   TGSI_OPCODE_IMSB               = 190,
-   TGSI_OPCODE_UMSB               = 191,
+   TGSI_OPCODE_IBFE,
+   TGSI_OPCODE_UBFE,
+   TGSI_OPCODE_BFI,
+   TGSI_OPCODE_BREV,
+   TGSI_OPCODE_POPC,
+   TGSI_OPCODE_LSB,
+   TGSI_OPCODE_IMSB,
+   TGSI_OPCODE_UMSB,
 
-   TGSI_OPCODE_INTERP_CENTROID    = 192,
-   TGSI_OPCODE_INTERP_SAMPLE      = 193,
-   TGSI_OPCODE_INTERP_OFFSET      = 194,
+   TGSI_OPCODE_INTERP_CENTROID,
+   TGSI_OPCODE_INTERP_SAMPLE,
+   TGSI_OPCODE_INTERP_OFFSET,
 
 /* sm5 marked opcodes are supported in D3D11 optionally - also DMOV, DMOVC */
-   TGSI_OPCODE_F2D                = 195 /* SM5 */,
-   TGSI_OPCODE_D2F                = 196,
-   TGSI_OPCODE_DABS               = 197,
-   TGSI_OPCODE_DNEG               = 198 /* SM5 */,
-   TGSI_OPCODE_DADD               = 199 /* SM5 */,
-   TGSI_OPCODE_DMUL               = 200 /* SM5 */,
-   TGSI_OPCODE_DMAX               = 201 /* SM5 */,
-   TGSI_OPCODE_DMIN               = 202 /* SM5 */,
-   TGSI_OPCODE_DSLT               = 203 /* SM5 */,
-   TGSI_OPCODE_DSGE               = 204 /* SM5 */,
-   TGSI_OPCODE_DSEQ               = 205 /* SM5 */,
-   TGSI_OPCODE_DSNE               = 206 /* SM5 */,
-   TGSI_OPCODE_DRCP               = 207 /* eg, cayman */,
-   TGSI_OPCODE_DSQRT              = 208 /* eg, cayman also has DRSQ */,
-   TGSI_OPCODE_DMAD               = 209,
-   TGSI_OPCODE_DFRAC              = 210 /* eg, cayman */,
-   TGSI_OPCODE_DLDEXP             = 211 /* eg, cayman */,
-   TGSI_OPCODE_DFRACEXP           = 212 /* eg, cayman */,
-   TGSI_OPCODE_D2I                = 213,
-   TGSI_OPCODE_I2D                = 214,
-   TGSI_OPCODE_D2U                = 215,
-   TGSI_OPCODE_U2D                = 216,
-   TGSI_OPCODE_DRSQ               = 217 /* eg, cayman also has DRSQ */,
-   TGSI_OPCODE_DTRUNC             = 218 /* nvc0 */,
-   TGSI_OPCODE_DCEIL              = 219 /* nvc0 */,
-   TGSI_OPCODE_DFLR               = 220 /* nvc0 */,
-   TGSI_OPCODE_DROUND             = 221 /* nvc0 */,
-   TGSI_OPCODE_DSSG               = 222,
-   TGSI_OPCODE_DDIV               = 223,
-   TGSI_OPCODE_CLOCK              = 224,
+   TGSI_OPCODE_F2D /* SM5 */,
+   TGSI_OPCODE_D2F,
+   TGSI_OPCODE_DABS,
+   TGSI_OPCODE_DNEG /* SM5 */,
+   TGSI_OPCODE_DADD /* SM5 */,
+   TGSI_OPCODE_DMUL /* SM5 */,
+   TGSI_OPCODE_DMAX /* SM5 */,
+   TGSI_OPCODE_DMIN /* SM5 */,
+   TGSI_OPCODE_DSLT /* SM5 */,
+   TGSI_OPCODE_DSGE /* SM5 */,
+   TGSI_OPCODE_DSEQ /* SM5 */,
+   TGSI_OPCODE_DSNE /* SM5 */,
+   TGSI_OPCODE_DRCP /* eg, cayman */,
+   TGSI_OPCODE_DSQRT /* eg, cayman also has DRSQ */,
+   TGSI_OPCODE_DMAD,
+   TGSI_OPCODE_DFRAC /* eg, cayman */,
+   TGSI_OPCODE_DLDEXP /* eg, cayman */,
+   TGSI_OPCODE_DFRACEXP /* eg, cayman */,
+   TGSI_OPCODE_D2I,
+   TGSI_OPCODE_I2D,
+   TGSI_OPCODE_D2U,
+   TGSI_OPCODE_U2D,
+   TGSI_OPCODE_DRSQ /* eg, cayman also has DRSQ */,
+   TGSI_OPCODE_DTRUNC /* nvc0 */,
+   TGSI_OPCODE_DCEIL /* nvc0 */,
+   TGSI_OPCODE_DFLR /* nvc0 */,
+   TGSI_OPCODE_DROUND /* nvc0 */,
+   TGSI_OPCODE_DSSG,
+   TGSI_OPCODE_DDIV,
+   TGSI_OPCODE_CLOCK,
 
 /* opcodes for ARB_gpu_shader_int64 */
-   TGSI_OPCODE_I64ABS             = 225,
-   TGSI_OPCODE_I64NEG             = 226,
-   TGSI_OPCODE_I64SSG             = 227,
-   TGSI_OPCODE_I64SLT             = 228,
-   TGSI_OPCODE_I64SGE             = 229,
-   TGSI_OPCODE_I64MIN             = 230,
-   TGSI_OPCODE_I64MAX             = 231,
-   TGSI_OPCODE_I64SHR             = 232,
-   TGSI_OPCODE_I64DIV             = 233,
-   TGSI_OPCODE_I64MOD             = 234,
-   TGSI_OPCODE_F2I64              = 235,
-   TGSI_OPCODE_U2I64              = 236,
-   TGSI_OPCODE_I2I64              = 237,
-   TGSI_OPCODE_D2I64              = 238,
-   TGSI_OPCODE_I642F              = 239,
-   TGSI_OPCODE_I642D              = 240,
+   TGSI_OPCODE_I64ABS,
+   TGSI_OPCODE_I64NEG,
+   TGSI_OPCODE_I64SSG,
+   TGSI_OPCODE_I64SLT,
+   TGSI_OPCODE_I64SGE,
+   TGSI_OPCODE_I64MIN,
+   TGSI_OPCODE_I64MAX,
+   TGSI_OPCODE_I64SHR,
+   TGSI_OPCODE_I64DIV,
+   TGSI_OPCODE_I64MOD,
+   TGSI_OPCODE_F2I64,
+   TGSI_OPCODE_U2I64,
+   TGSI_OPCODE_I2I64,
+   TGSI_OPCODE_D2I64,
+   TGSI_OPCODE_I642F,
+   TGSI_OPCODE_I642D,
 
-   TGSI_OPCODE_U64ADD             = 241,
-   TGSI_OPCODE_U64MUL             = 242,
-   TGSI_OPCODE_U64SEQ             = 243,
-   TGSI_OPCODE_U64SNE             = 244,
-   TGSI_OPCODE_U64SLT             = 245,
-   TGSI_OPCODE_U64SGE             = 246,
-   TGSI_OPCODE_U64MIN             = 247,
-   TGSI_OPCODE_U64MAX             = 248,
-   TGSI_OPCODE_U64SHL             = 249,
-   TGSI_OPCODE_U64SHR             = 250,
-   TGSI_OPCODE_U64DIV             = 251,
-   TGSI_OPCODE_U64MOD             = 252,
-   TGSI_OPCODE_F2U64              = 253,
-   TGSI_OPCODE_D2U64              = 254,
-   TGSI_OPCODE_U642F              = 255,
-   TGSI_OPCODE_U642D              = 256,
+   TGSI_OPCODE_U64ADD,
+   TGSI_OPCODE_U64MUL,
+   TGSI_OPCODE_U64SEQ,
+   TGSI_OPCODE_U64SNE,
+   TGSI_OPCODE_U64SLT,
+   TGSI_OPCODE_U64SGE,
+   TGSI_OPCODE_U64MIN,
+   TGSI_OPCODE_U64MAX,
+   TGSI_OPCODE_U64SHL,
+   TGSI_OPCODE_U64SHR,
+   TGSI_OPCODE_U64DIV,
+   TGSI_OPCODE_U64MOD,
+   TGSI_OPCODE_F2U64,
+   TGSI_OPCODE_D2U64,
+   TGSI_OPCODE_U642F,
+   TGSI_OPCODE_U642D,
 
-   TGSI_OPCODE_LAST               = 257,
+   TGSI_OPCODE_LAST,
 };
 
 /**
