@@ -300,6 +300,7 @@ proxy_context_submit_cmd(struct virgl_context *base, const void *buffer, size_t
 static bool
 validate_resource_fd_shm(int fd, uint64_t expected_size)
 {
+#if defined(HAVE_MEMFD_CREATE)
    static const int blocked_seals = F_SEAL_WRITE;
 
    const int seals = fcntl(fd, F_GET_SEALS);
@@ -307,6 +308,7 @@ validate_resource_fd_shm(int fd, uint64_t expected_size)
       proxy_log("failed to validate shm seals(%d): blocked(%d)", seals, blocked_seals);
       return false;
    }
+#endif
 
    const uint64_t size = lseek(fd, 0, SEEK_END);
    if (size != expected_size) {
@@ -321,7 +323,13 @@ validate_resource_fd_shm(int fd, uint64_t expected_size)
 static inline int
 add_required_seals_to_fd(int fd)
 {
+#if defined(HAVE_MEMFD_CREATE)
    return fcntl(fd, F_ADD_SEALS, F_SEAL_SEAL | F_SEAL_SHRINK | F_SEAL_GROW);
+#else
+   // if memfd_create is not available pretend file sealing worked
+   (void)fd;
+   return 0;
+#endif
 }
 
 static int
@@ -442,28 +450,32 @@ proxy_context_attach_resource(struct virgl_context *base, struct virgl_resource
    if (proxy_context_resource_find(ctx, res_id))
       return;
 
-   /* The current render protocol only supports importing dma-buf or pipe resource that
-    * can be exported to dma-buf. A protocol change is needed when there exists use case
-    * for importing external Vulkan opaque resource. For shm, we only create with blob_id
-    * 0 via CREATE_RESOURCE above.
+   /* The current render protocol only supports importing dma-buf, shm or pipe resource
+    * that can be exported to dma-buf. A protocol change is needed when there exists use
+    * case for importing external Vulkan opaque resource.
     */
-   assert(res->fd_type == VIRGL_RESOURCE_FD_INVALID ||
-          res->fd_type == VIRGL_RESOURCE_FD_DMABUF);
+   if (res->fd_type != VIRGL_RESOURCE_FD_INVALID &&
+       res->fd_type != VIRGL_RESOURCE_FD_DMABUF &&
+       res->fd_type != VIRGL_RESOURCE_FD_SHM) {
+      proxy_log("failed to attach res %d with fd_type %d", res_id, res->fd_type);
+      return;
+   }
 
    enum virgl_resource_fd_type res_fd_type = res->fd_type;
    int res_fd = res->fd;
    uint64_t res_size = res->map_size;
    bool close_res_fd = false;
    if (res_fd_type == VIRGL_RESOURCE_FD_INVALID) {
+      /* importable pipe resouce can only export as dma-buf */
       res_fd_type = virgl_resource_export_fd(res, &res_fd);
-      if (res_fd_type == VIRGL_RESOURCE_FD_INVALID) {
-         proxy_log("failed to export res %d", res_id);
+      if (res_fd_type != VIRGL_RESOURCE_FD_DMABUF) {
+         /* close fd for unexpected fd type from succeeded export */
+         if (res_fd_type != VIRGL_RESOURCE_FD_INVALID)
+            close(res_fd);
+         proxy_log("exported res %d to unexpected fd_type %d", res_id, res_fd_type);
          return;
       }
 
-      /* importable pipe resouce can only export as dma-buf */
-      assert(res_fd_type == VIRGL_RESOURCE_FD_DMABUF);
-
       /* get the actual dma-buf size here because:
        * - pipe resource created by vrend has a zero map_size
        * - blob resource created by vrend can have non-zero fake map_size
@@ -641,8 +653,10 @@ static bool
 proxy_context_init(struct proxy_context *ctx, uint32_t ctx_flags)
 {
    if (!proxy_context_init_shmem(ctx) || !proxy_context_init_timelines(ctx) ||
-       !proxy_context_init_fencing(ctx) || !proxy_context_resource_table_init(ctx))
+       !proxy_context_init_fencing(ctx) || !proxy_context_resource_table_init(ctx)) {
+      proxy_log("failed to pre-initialize context");
       return false;
+   }
 
    const struct render_context_op_init_request req = {
       .header.op = RENDER_CONTEXT_OP_INIT,
