@@ -24,6 +24,7 @@ struct vkr_device {
    mtx_t free_sync_mutex;
    struct list_head free_syncs;
 
+   mtx_t object_mutex;
    struct list_head objects;
 };
 VKR_DEFINE_OBJECT_CAST(device, VK_OBJECT_TYPE_DEVICE, VkDevice)
@@ -32,7 +33,7 @@ void
 vkr_context_init_device_dispatch(struct vkr_context *ctx);
 
 void
-vkr_device_destroy(struct vkr_context *ctx, struct vkr_device *dev);
+vkr_device_destroy(struct vkr_context *ctx, struct vkr_device *dev, bool destroy_vk);
 
 static inline bool
 vkr_device_should_track_object(const struct vkr_object *obj)
@@ -60,7 +61,10 @@ vkr_device_add_object(struct vkr_context *ctx,
    vkr_context_add_object(ctx, obj);
 
    assert(vkr_device_should_track_object(obj));
+
+   mtx_lock(&dev->object_mutex);
    list_add(&obj->track_head, &dev->objects);
+   mtx_unlock(&dev->object_mutex);
 }
 
 static inline void
@@ -69,7 +73,10 @@ vkr_device_remove_object(struct vkr_context *ctx,
                          struct vkr_object *obj)
 {
    assert(vkr_device_should_track_object(obj));
+
+   mtx_lock(&dev->object_mutex);
    list_del(&obj->track_head);
+   mtx_unlock(&dev->object_mutex);
 
    /* this frees obj */
    vkr_context_remove_object(ctx, obj);
