@@ -104,9 +104,10 @@ vkr_instance_enumerate_physical_devices(struct vkr_instance *instance)
    if (instance->physical_device_count)
       return VK_SUCCESS;
 
+   struct vn_instance_proc_table *vk = &instance->proc_table;
    uint32_t count;
    VkResult result =
-      vkEnumeratePhysicalDevices(instance->base.handle.instance, &count, NULL);
+      vk->EnumeratePhysicalDevices(instance->base.handle.instance, &count, NULL);
    if (result != VK_SUCCESS)
       return result;
 
@@ -118,7 +119,7 @@ vkr_instance_enumerate_physical_devices(struct vkr_instance *instance)
       return VK_ERROR_OUT_OF_HOST_MEMORY;
    }
 
-   result = vkEnumeratePhysicalDevices(instance->base.handle.instance, &count, handles);
+   result = vk->EnumeratePhysicalDevices(instance->base.handle.instance, &count, handles);
    if (result != VK_SUCCESS) {
       free(physical_devs);
       free(handles);
@@ -151,20 +152,24 @@ vkr_instance_lookup_physical_device(struct vkr_instance *instance,
 static void
 vkr_physical_device_init_id_properties(struct vkr_physical_device *physical_dev)
 {
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    VkPhysicalDevice handle = physical_dev->base.handle.physical_device;
    physical_dev->id_properties.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ID_PROPERTIES;
    VkPhysicalDeviceProperties2 props2 = {
       .sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2,
       .pNext = &physical_dev->id_properties
    };
-   vkGetPhysicalDeviceProperties2(handle, &props2);
+   vk->GetPhysicalDeviceProperties2(handle, &props2);
 }
 
 static void
 vkr_physical_device_init_memory_properties(struct vkr_physical_device *physical_dev)
 {
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    VkPhysicalDevice handle = physical_dev->base.handle.physical_device;
-   vkGetPhysicalDeviceMemoryProperties(handle, &physical_dev->memory_properties);
+   vk->GetPhysicalDeviceMemoryProperties(handle, &physical_dev->memory_properties);
 
    /* XXX When a VkMemoryType has VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT, we
     * assume any VkDeviceMemory with the memory type can be made external and
@@ -204,7 +209,7 @@ vkr_physical_device_init_memory_properties(struct vkr_physical_device *physical_
 
    if (physical_dev->EXT_external_memory_dma_buf) {
       info.handleType = VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT,
-      vkGetPhysicalDeviceExternalBufferProperties(handle, &info, &props);
+      vk->GetPhysicalDeviceExternalBufferProperties(handle, &info, &props);
       physical_dev->is_dma_buf_fd_export_supported =
          (props.externalMemoryProperties.externalMemoryFeatures &
           VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT) &&
@@ -214,7 +219,7 @@ vkr_physical_device_init_memory_properties(struct vkr_physical_device *physical_
 
    if (physical_dev->KHR_external_memory_fd) {
       info.handleType = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT,
-      vkGetPhysicalDeviceExternalBufferProperties(handle, &info, &props);
+      vk->GetPhysicalDeviceExternalBufferProperties(handle, &info, &props);
       physical_dev->is_opaque_fd_export_supported =
          (props.externalMemoryProperties.externalMemoryFeatures &
           VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT) &&
@@ -222,23 +227,31 @@ vkr_physical_device_init_memory_properties(struct vkr_physical_device *physical_
           VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT);
    }
 
+   /* fallback to gbm allocation with dma-buf import */
    if (!physical_dev->is_dma_buf_fd_export_supported &&
-       !physical_dev->is_opaque_fd_export_supported)
+       !physical_dev->is_opaque_fd_export_supported &&
+       physical_dev->EXT_external_memory_dma_buf)
       physical_dev->gbm_device = vkr_physical_device_get_gbm_device();
 
-   physical_dev->udmabuf_dev_fd =
-      VKR_DEBUG(UDMABUF) ? vkr_physical_device_get_udmabuf_dev_fd() : -1;
+   physical_dev->udmabuf_dev_fd = -1;
+   if (VKR_DEBUG(UDMABUF)) {
+      if (physical_dev->EXT_external_memory_dma_buf)
+         physical_dev->udmabuf_dev_fd = vkr_physical_device_get_udmabuf_dev_fd();
+      else
+         vkr_log("missing VK_EXT_external_memory_dma_buf for udmabuf import!");
+   }
 }
 
 static void
-vkr_physical_device_init_extensions(struct vkr_physical_device *physical_dev,
-                                    struct vkr_instance *instance)
+vkr_physical_device_init_extensions(struct vkr_physical_device *physical_dev)
 {
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    VkPhysicalDevice handle = physical_dev->base.handle.physical_device;
 
    VkExtensionProperties *exts;
    uint32_t count;
-   VkResult result = vkEnumerateDeviceExtensionProperties(handle, NULL, &count, NULL);
+   VkResult result = vk->EnumerateDeviceExtensionProperties(handle, NULL, &count, NULL);
    if (result != VK_SUCCESS)
       return;
 
@@ -246,7 +259,7 @@ vkr_physical_device_init_extensions(struct vkr_physical_device *physical_dev,
    if (!exts)
       return;
 
-   result = vkEnumerateDeviceExtensionProperties(handle, NULL, &count, exts);
+   result = vk->EnumerateDeviceExtensionProperties(handle, NULL, &count, exts);
    if (result != VK_SUCCESS) {
       free(exts);
       return;
@@ -279,10 +292,7 @@ vkr_physical_device_init_extensions(struct vkr_physical_device *physical_dev,
       VkExternalFenceProperties fence_props = {
          .sType = VK_STRUCTURE_TYPE_EXTERNAL_FENCE_PROPERTIES,
       };
-      PFN_vkGetPhysicalDeviceExternalFenceProperties get_fence_props =
-         (PFN_vkGetPhysicalDeviceExternalFenceProperties)vkGetInstanceProcAddr(
-            instance->base.handle.instance, "vkGetPhysicalDeviceExternalFenceProperties");
-      get_fence_props(handle, &fence_info, &fence_props);
+      vk->GetPhysicalDeviceExternalFenceProperties(handle, &fence_info, &fence_props);
 
       if (!(fence_props.externalFenceFeatures & VK_EXTERNAL_FENCE_FEATURE_EXPORTABLE_BIT))
          physical_dev->KHR_external_fence_fd = false;
@@ -295,35 +305,39 @@ vkr_physical_device_init_extensions(struct vkr_physical_device *physical_dev,
 static void
 vkr_physical_device_init_properties(struct vkr_physical_device *physical_dev)
 {
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    VkPhysicalDevice handle = physical_dev->base.handle.physical_device;
-   vkGetPhysicalDeviceProperties(handle, &physical_dev->properties);
+   vk->GetPhysicalDeviceProperties(handle, &physical_dev->properties);
 
    VkPhysicalDeviceProperties *props = &physical_dev->properties;
    props->apiVersion = vkr_api_version_cap_minor(props->apiVersion, VKR_MAX_API_VERSION);
 }
 
-static void
+static inline void
 vkr_physical_device_init_proc_table(struct vkr_physical_device *physical_dev,
                                     struct vkr_instance *instance)
 {
-   vn_util_init_physical_device_proc_table(instance->base.handle.instance,
-                                           &physical_dev->proc_table);
+   vn_util_init_physical_device_proc_table(
+      instance->base.handle.instance, instance->get_proc_addr, &physical_dev->proc_table);
 }
 
 static void
 vkr_physical_device_init_queue_family_properties(struct vkr_physical_device *physical_dev)
 {
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    VkPhysicalDevice handle = physical_dev->base.handle.physical_device;
 
    VkQueueFamilyProperties *props;
    uint32_t count;
-   vkGetPhysicalDeviceQueueFamilyProperties(handle, &count, NULL);
+   vk->GetPhysicalDeviceQueueFamilyProperties(handle, &count, NULL);
 
    props = malloc(sizeof(*props) * count);
    if (!props)
       return;
 
-   vkGetPhysicalDeviceQueueFamilyProperties(handle, &count, props);
+   vk->GetPhysicalDeviceQueueFamilyProperties(handle, &count, props);
 
    physical_dev->queue_family_property_count = count;
    physical_dev->queue_family_properties = props;
@@ -390,7 +404,7 @@ vkr_dispatch_vkEnumeratePhysicalDevices(struct vn_dispatch_context *dispatch,
       vkr_physical_device_init_properties(physical_dev);
       physical_dev->api_version =
          MIN2(physical_dev->properties.apiVersion, instance->api_version);
-      vkr_physical_device_init_extensions(physical_dev, instance);
+      vkr_physical_device_init_extensions(physical_dev);
       vkr_physical_device_init_memory_properties(physical_dev);
       vkr_physical_device_init_id_properties(physical_dev);
       vkr_physical_device_init_queue_family_properties(physical_dev);
@@ -447,10 +461,11 @@ vkr_dispatch_vkEnumeratePhysicalDeviceGroups(
       }
    }
 
+   struct vn_instance_proc_table *vk = &instance->proc_table;
    vn_replace_vkEnumeratePhysicalDeviceGroups_args_handle(args);
    args->ret =
-      vkEnumeratePhysicalDeviceGroups(args->instance, args->pPhysicalDeviceGroupCount,
-                                      args->pPhysicalDeviceGroupProperties);
+      vk->EnumeratePhysicalDeviceGroups(args->instance, args->pPhysicalDeviceGroupCount,
+                                        args->pPhysicalDeviceGroupProperties);
    if (args->ret != VK_SUCCESS)
       return;
 
@@ -524,8 +539,12 @@ vkr_dispatch_vkGetPhysicalDeviceFeatures(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceFeatures *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceFeatures_args_handle(args);
-   vkGetPhysicalDeviceFeatures(args->physicalDevice, args->pFeatures);
+   vk->GetPhysicalDeviceFeatures(args->physicalDevice, args->pFeatures);
 }
 
 static void
@@ -544,10 +563,14 @@ vkr_dispatch_vkGetPhysicalDeviceQueueFamilyProperties(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceQueueFamilyProperties *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceQueueFamilyProperties_args_handle(args);
-   vkGetPhysicalDeviceQueueFamilyProperties(args->physicalDevice,
-                                            args->pQueueFamilyPropertyCount,
-                                            args->pQueueFamilyProperties);
+   vk->GetPhysicalDeviceQueueFamilyProperties(args->physicalDevice,
+                                              args->pQueueFamilyPropertyCount,
+                                              args->pQueueFamilyProperties);
 }
 
 static void
@@ -565,9 +588,13 @@ vkr_dispatch_vkGetPhysicalDeviceFormatProperties(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceFormatProperties *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceFormatProperties_args_handle(args);
-   vkGetPhysicalDeviceFormatProperties(args->physicalDevice, args->format,
-                                       args->pFormatProperties);
+   vk->GetPhysicalDeviceFormatProperties(args->physicalDevice, args->format,
+                                         args->pFormatProperties);
 }
 
 static void
@@ -575,8 +602,12 @@ vkr_dispatch_vkGetPhysicalDeviceImageFormatProperties(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceImageFormatProperties *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceImageFormatProperties_args_handle(args);
-   args->ret = vkGetPhysicalDeviceImageFormatProperties(
+   args->ret = vk->GetPhysicalDeviceImageFormatProperties(
       args->physicalDevice, args->format, args->type, args->tiling, args->usage,
       args->flags, args->pImageFormatProperties);
 }
@@ -586,8 +617,12 @@ vkr_dispatch_vkGetPhysicalDeviceSparseImageFormatProperties(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceSparseImageFormatProperties *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceSparseImageFormatProperties_args_handle(args);
-   vkGetPhysicalDeviceSparseImageFormatProperties(
+   vk->GetPhysicalDeviceSparseImageFormatProperties(
       args->physicalDevice, args->format, args->type, args->samples, args->usage,
       args->tiling, args->pPropertyCount, args->pProperties);
 }
@@ -597,8 +632,12 @@ vkr_dispatch_vkGetPhysicalDeviceFeatures2(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceFeatures2 *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceFeatures2_args_handle(args);
-   vkGetPhysicalDeviceFeatures2(args->physicalDevice, args->pFeatures);
+   vk->GetPhysicalDeviceFeatures2(args->physicalDevice, args->pFeatures);
 }
 
 static void
@@ -606,8 +645,12 @@ vkr_dispatch_vkGetPhysicalDeviceProperties2(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceProperties2 *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceProperties2_args_handle(args);
-   vkGetPhysicalDeviceProperties2(args->physicalDevice, args->pProperties);
+   vk->GetPhysicalDeviceProperties2(args->physicalDevice, args->pProperties);
 }
 
 static void
@@ -615,10 +658,14 @@ vkr_dispatch_vkGetPhysicalDeviceQueueFamilyProperties2(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceQueueFamilyProperties2 *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceQueueFamilyProperties2_args_handle(args);
-   vkGetPhysicalDeviceQueueFamilyProperties2(args->physicalDevice,
-                                             args->pQueueFamilyPropertyCount,
-                                             args->pQueueFamilyProperties);
+   vk->GetPhysicalDeviceQueueFamilyProperties2(args->physicalDevice,
+                                               args->pQueueFamilyPropertyCount,
+                                               args->pQueueFamilyProperties);
 }
 
 static void
@@ -628,6 +675,7 @@ vkr_dispatch_vkGetPhysicalDeviceMemoryProperties2(
 {
    struct vkr_physical_device *physical_dev =
       vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
 
    if (args->pMemoryProperties->pNext == NULL) {
       /* The client is querying only VkPhysicalDeviceMemoryProperties, which is
@@ -636,8 +684,8 @@ vkr_dispatch_vkGetPhysicalDeviceMemoryProperties2(
       args->pMemoryProperties->memoryProperties = physical_dev->memory_properties;
    } else {
       vn_replace_vkGetPhysicalDeviceMemoryProperties2_args_handle(args);
-      vkGetPhysicalDeviceMemoryProperties2(args->physicalDevice,
-                                           args->pMemoryProperties);
+      vk->GetPhysicalDeviceMemoryProperties2(args->physicalDevice,
+                                             args->pMemoryProperties);
    }
 }
 
@@ -646,9 +694,13 @@ vkr_dispatch_vkGetPhysicalDeviceFormatProperties2(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceFormatProperties2 *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceFormatProperties2_args_handle(args);
-   vkGetPhysicalDeviceFormatProperties2(args->physicalDevice, args->format,
-                                        args->pFormatProperties);
+   vk->GetPhysicalDeviceFormatProperties2(args->physicalDevice, args->format,
+                                          args->pFormatProperties);
 }
 
 static void
@@ -656,8 +708,12 @@ vkr_dispatch_vkGetPhysicalDeviceImageFormatProperties2(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceImageFormatProperties2 *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceImageFormatProperties2_args_handle(args);
-   args->ret = vkGetPhysicalDeviceImageFormatProperties2(
+   args->ret = vk->GetPhysicalDeviceImageFormatProperties2(
       args->physicalDevice, args->pImageFormatInfo, args->pImageFormatProperties);
 }
 
@@ -666,8 +722,12 @@ vkr_dispatch_vkGetPhysicalDeviceSparseImageFormatProperties2(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceSparseImageFormatProperties2 *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceSparseImageFormatProperties2_args_handle(args);
-   vkGetPhysicalDeviceSparseImageFormatProperties2(
+   vk->GetPhysicalDeviceSparseImageFormatProperties2(
       args->physicalDevice, args->pFormatInfo, args->pPropertyCount, args->pProperties);
 }
 
@@ -676,8 +736,12 @@ vkr_dispatch_vkGetPhysicalDeviceExternalBufferProperties(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceExternalBufferProperties *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceExternalBufferProperties_args_handle(args);
-   vkGetPhysicalDeviceExternalBufferProperties(
+   vk->GetPhysicalDeviceExternalBufferProperties(
       args->physicalDevice, args->pExternalBufferInfo, args->pExternalBufferProperties);
 }
 
@@ -686,10 +750,14 @@ vkr_dispatch_vkGetPhysicalDeviceExternalSemaphoreProperties(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceExternalSemaphoreProperties *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceExternalSemaphoreProperties_args_handle(args);
-   vkGetPhysicalDeviceExternalSemaphoreProperties(args->physicalDevice,
-                                                  args->pExternalSemaphoreInfo,
-                                                  args->pExternalSemaphoreProperties);
+   vk->GetPhysicalDeviceExternalSemaphoreProperties(args->physicalDevice,
+                                                    args->pExternalSemaphoreInfo,
+                                                    args->pExternalSemaphoreProperties);
 }
 
 static void
@@ -697,22 +765,26 @@ vkr_dispatch_vkGetPhysicalDeviceExternalFenceProperties(
    UNUSED struct vn_dispatch_context *dispatch,
    struct vn_command_vkGetPhysicalDeviceExternalFenceProperties *args)
 {
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
    vn_replace_vkGetPhysicalDeviceExternalFenceProperties_args_handle(args);
-   vkGetPhysicalDeviceExternalFenceProperties(
+   vk->GetPhysicalDeviceExternalFenceProperties(
       args->physicalDevice, args->pExternalFenceInfo, args->pExternalFenceProperties);
 }
 
 static void
-vkr_dispatch_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT(
+vkr_dispatch_vkGetPhysicalDeviceCalibrateableTimeDomainsKHR(
    UNUSED struct vn_dispatch_context *ctx,
-   struct vn_command_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT *args)
+   struct vn_command_vkGetPhysicalDeviceCalibrateableTimeDomainsKHR *args)
 {
    struct vkr_physical_device *physical_dev =
       vkr_physical_device_from_handle(args->physicalDevice);
    struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
 
-   vn_replace_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT_args_handle(args);
-   args->ret = vk->GetPhysicalDeviceCalibrateableTimeDomainsEXT(
+   vn_replace_vkGetPhysicalDeviceCalibrateableTimeDomainsKHR_args_handle(args);
+   args->ret = vk->GetPhysicalDeviceCalibrateableTimeDomainsKHR(
       args->physicalDevice, args->pTimeDomainCount, args->pTimeDomains);
 }
 
@@ -730,6 +802,20 @@ vkr_dispatch_vkGetPhysicalDeviceFragmentShadingRatesKHR(
       args->physicalDevice, args->pFragmentShadingRateCount, args->pFragmentShadingRates);
 }
 
+static void
+vkr_dispatch_vkGetPhysicalDeviceMultisamplePropertiesEXT(
+   UNUSED struct vn_dispatch_context *ctx,
+   struct vn_command_vkGetPhysicalDeviceMultisamplePropertiesEXT *args)
+{
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
+   vn_replace_vkGetPhysicalDeviceMultisamplePropertiesEXT_args_handle(args);
+   vk->GetPhysicalDeviceMultisamplePropertiesEXT(args->physicalDevice, args->samples,
+                                                 args->pMultisampleProperties);
+}
+
 void
 vkr_context_init_physical_device_dispatch(struct vkr_context *ctx)
 {
@@ -779,10 +865,14 @@ vkr_context_init_physical_device_dispatch(struct vkr_context *ctx)
       vkr_dispatch_vkGetPhysicalDeviceExternalSemaphoreProperties;
    dispatch->dispatch_vkGetPhysicalDeviceExternalFenceProperties =
       vkr_dispatch_vkGetPhysicalDeviceExternalFenceProperties;
-   dispatch->dispatch_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT =
-      vkr_dispatch_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT;
+   dispatch->dispatch_vkGetPhysicalDeviceCalibrateableTimeDomainsKHR =
+      vkr_dispatch_vkGetPhysicalDeviceCalibrateableTimeDomainsKHR;
 
    /* VK_KHR_fragment_shading_rate */
    dispatch->dispatch_vkGetPhysicalDeviceFragmentShadingRatesKHR =
       vkr_dispatch_vkGetPhysicalDeviceFragmentShadingRatesKHR;
+
+   /* VK_EXT_sample_locations */
+   dispatch->dispatch_vkGetPhysicalDeviceMultisamplePropertiesEXT =
+      vkr_dispatch_vkGetPhysicalDeviceMultisamplePropertiesEXT;
 }
