@@ -11,7 +11,7 @@
 #include "vkr_physical_device.h"
 #include "vkr_queue_gen.h"
 
-struct vkr_queue_sync *
+static struct vkr_queue_sync *
 vkr_device_alloc_queue_sync(struct vkr_device *dev,
                             uint32_t fence_flags,
                             uint32_t ring_idx,
@@ -20,12 +20,9 @@ vkr_device_alloc_queue_sync(struct vkr_device *dev,
    struct vn_device_proc_table *vk = &dev->proc_table;
    struct vkr_queue_sync *sync;
 
-   if (vkr_renderer_flags & VKR_RENDERER_ASYNC_FENCE_CB)
-      mtx_lock(&dev->free_sync_mutex);
-
-   if (LIST_IS_EMPTY(&dev->free_syncs)) {
-      if (vkr_renderer_flags & VKR_RENDERER_ASYNC_FENCE_CB)
-         mtx_unlock(&dev->free_sync_mutex);
+   mtx_lock(&dev->free_sync_mutex);
+   if (list_is_empty(&dev->free_syncs)) {
+      mtx_unlock(&dev->free_sync_mutex);
 
       sync = malloc(sizeof(*sync));
       if (!sync)
@@ -43,14 +40,13 @@ vkr_device_alloc_queue_sync(struct vkr_device *dev,
          vk->CreateFence(dev->base.handle.device, &create_info, NULL, &sync->fence);
       if (result != VK_SUCCESS) {
          free(sync);
+         vkr_log("failed to create sync fence for fence_id %" PRIu64, fence_id);
          return NULL;
       }
    } else {
       sync = LIST_ENTRY(struct vkr_queue_sync, dev->free_syncs.next, head);
       list_del(&sync->head);
-
-      if (vkr_renderer_flags & VKR_RENDERER_ASYNC_FENCE_CB)
-         mtx_unlock(&dev->free_sync_mutex);
+      mtx_unlock(&dev->free_sync_mutex);
 
       vk->ResetFences(dev->base.handle.device, 1, &sync->fence);
    }
@@ -63,121 +59,85 @@ vkr_device_alloc_queue_sync(struct vkr_device *dev,
    return sync;
 }
 
-void
+static void
 vkr_device_free_queue_sync(struct vkr_device *dev, struct vkr_queue_sync *sync)
 {
-   if (vkr_renderer_flags & VKR_RENDERER_ASYNC_FENCE_CB) {
-      mtx_lock(&dev->free_sync_mutex);
-      list_addtail(&sync->head, &dev->free_syncs);
-      mtx_unlock(&dev->free_sync_mutex);
-   } else {
-      list_addtail(&sync->head, &dev->free_syncs);
-   }
+   mtx_lock(&dev->free_sync_mutex);
+   list_addtail(&sync->head, &dev->free_syncs);
+   mtx_unlock(&dev->free_sync_mutex);
 }
 
-void
-vkr_queue_get_signaled_syncs(struct vkr_queue *queue,
-                             struct list_head *retired_syncs,
-                             bool *queue_empty)
+static inline void
+vkr_queue_sync_retire(struct vkr_queue *queue, struct vkr_queue_sync *sync)
 {
+   TRACE_FUNC();
+   queue->context->retire_fence(queue->context->ctx_id, sync->ring_idx, sync->fence_id);
+   vkr_device_free_queue_sync(queue->device, sync);
+}
+
+bool
+vkr_queue_sync_submit(struct vkr_queue *queue,
+                      uint32_t flags,
+                      uint32_t ring_idx,
+                      uint64_t fence_id)
+{
+   TRACE_FUNC();
    struct vkr_device *dev = queue->device;
    struct vn_device_proc_table *vk = &dev->proc_table;
-   struct vkr_queue_sync *sync, *tmp;
 
-   assert(!(vkr_renderer_flags & VKR_RENDERER_ASYNC_FENCE_CB));
+   struct vkr_queue_sync *sync =
+      vkr_device_alloc_queue_sync(dev, flags, ring_idx, fence_id);
+   if (!sync)
+      return false;
 
-   list_inithead(retired_syncs);
+   mtx_lock(&queue->vk_mutex);
+   VkResult result = vk->QueueSubmit(queue->base.handle.queue, 0, NULL, sync->fence);
+   mtx_unlock(&queue->vk_mutex);
 
-   if (vkr_renderer_flags & VKR_RENDERER_THREAD_SYNC) {
-      mtx_lock(&queue->mutex);
-
-      LIST_FOR_EACH_ENTRY_SAFE (sync, tmp, &queue->signaled_syncs, head) {
-         if (sync->head.next == &queue->signaled_syncs ||
-             !(sync->flags & VIRGL_RENDERER_FENCE_FLAG_MERGEABLE))
-            list_addtail(&sync->head, retired_syncs);
-         else
-            vkr_device_free_queue_sync(dev, sync);
-      }
-      list_inithead(&queue->signaled_syncs);
-
-      *queue_empty = LIST_IS_EMPTY(&queue->pending_syncs);
-
-      mtx_unlock(&queue->mutex);
-   } else {
-      LIST_FOR_EACH_ENTRY_SAFE (sync, tmp, &queue->pending_syncs, head) {
-         if (!sync->device_lost) {
-            VkResult result = vk->GetFenceStatus(dev->base.handle.device, sync->fence);
-            if (result == VK_NOT_READY)
-               break;
-         }
-
-         bool is_last_sync = sync->head.next == &queue->pending_syncs;
-
-         list_del(&sync->head);
-         if (is_last_sync || !(sync->flags & VIRGL_RENDERER_FENCE_FLAG_MERGEABLE))
-            list_addtail(&sync->head, retired_syncs);
-         else
-            vkr_device_free_queue_sync(dev, sync);
-      }
-
-      *queue_empty = LIST_IS_EMPTY(&queue->pending_syncs);
-   }
-}
-
-static void
-vkr_queue_sync_retire(struct vkr_context *ctx,
-                      struct vkr_device *dev,
-                      struct vkr_queue_sync *sync)
-{
-   struct vn_device_proc_table *vk = &dev->proc_table;
-
-   if (vkr_renderer_flags & VKR_RENDERER_ASYNC_FENCE_CB) {
-      ctx->base.fence_retire(&ctx->base, sync->ring_idx, sync->fence_id);
+   if (result == VK_ERROR_DEVICE_LOST) {
+      sync->device_lost = true;
+      vkr_log("sync submit hit device lost for fence_id %" PRIu64, fence_id);
+   } else if (result != VK_SUCCESS) {
       vkr_device_free_queue_sync(dev, sync);
-   } else {
-      vk->DestroyFence(dev->base.handle.device, sync->fence, NULL);
-      sync->fence = VK_NULL_HANDLE;
-
-      /* move to the ctx to be retired and freed at the next retire_fences */
-      list_addtail(&sync->head, &ctx->signaled_syncs);
+      vkr_log("sync submit failed (vk ret %d) for fence_id %" PRIu64, result, fence_id);
+      return false;
    }
+
+   mtx_lock(&queue->sync_thread.mutex);
+   list_addtail(&sync->head, &queue->sync_thread.syncs);
+   cnd_signal(&queue->sync_thread.cond);
+   mtx_unlock(&queue->sync_thread.mutex);
+
+   return true;
 }
 
 static void
-vkr_queue_retire_all_syncs(struct vkr_context *ctx, struct vkr_queue *queue)
+vkr_queue_sync_thread_fini(struct vkr_queue *queue)
 {
-   struct vkr_queue_sync *sync, *tmp;
+   /* vkDeviceWaitIdle has been called */
+   mtx_lock(&queue->sync_thread.mutex);
+   queue->sync_thread.join = true;
+   cnd_signal(&queue->sync_thread.cond);
+   mtx_unlock(&queue->sync_thread.mutex);
 
-   if (vkr_renderer_flags & VKR_RENDERER_THREAD_SYNC) {
-      mtx_lock(&queue->mutex);
-      queue->join = true;
-      mtx_unlock(&queue->mutex);
+   thrd_join(queue->sync_thread.thread, NULL);
 
-      cnd_signal(&queue->cond);
-      thrd_join(queue->thread, NULL);
+   list_for_each_entry_safe (struct vkr_queue_sync, sync, &queue->sync_thread.syncs, head)
+      vkr_queue_sync_retire(queue, sync);
 
-      LIST_FOR_EACH_ENTRY_SAFE (sync, tmp, &queue->signaled_syncs, head)
-         vkr_queue_sync_retire(ctx, queue->device, sync);
-   } else {
-      assert(LIST_IS_EMPTY(&queue->signaled_syncs));
-   }
-
-   LIST_FOR_EACH_ENTRY_SAFE (sync, tmp, &queue->pending_syncs, head)
-      vkr_queue_sync_retire(ctx, queue->device, sync);
+   mtx_destroy(&queue->sync_thread.mutex);
+   cnd_destroy(&queue->sync_thread.cond);
 }
 
 void
 vkr_queue_destroy(struct vkr_context *ctx, struct vkr_queue *queue)
 {
-   /* vkDeviceWaitIdle has been called */
-   vkr_queue_retire_all_syncs(ctx, queue);
+   vkr_queue_sync_thread_fini(queue);
 
-   mtx_destroy(&queue->mutex);
-   cnd_destroy(&queue->cond);
-
-   list_del(&queue->busy_head);
    list_del(&queue->base.track_head);
 
+   mtx_destroy(&queue->vk_mutex);
+
    if (queue->ring_idx > 0)
       ctx->sync_queues[queue->ring_idx] = NULL;
 
@@ -197,21 +157,21 @@ vkr_queue_thread(void *arg)
    const uint64_t ns_per_sec = 1000000000llu;
    char thread_name[16];
 
-   snprintf(thread_name, ARRAY_SIZE(thread_name), "vkr-queue-%d", ctx->base.ctx_id);
+   snprintf(thread_name, ARRAY_SIZE(thread_name), "vkr-queue-%d", ctx->ctx_id);
    u_thread_setname(thread_name);
 
-   mtx_lock(&queue->mutex);
+   mtx_lock(&queue->sync_thread.mutex);
    while (true) {
-      while (LIST_IS_EMPTY(&queue->pending_syncs) && !queue->join)
-         cnd_wait(&queue->cond, &queue->mutex);
+      while (list_is_empty(&queue->sync_thread.syncs) && !queue->sync_thread.join)
+         cnd_wait(&queue->sync_thread.cond, &queue->sync_thread.mutex);
 
-      if (queue->join)
+      if (queue->sync_thread.join)
          break;
 
       struct vkr_queue_sync *sync =
-         LIST_ENTRY(struct vkr_queue_sync, queue->pending_syncs.next, head);
+         list_first_entry(&queue->sync_thread.syncs, struct vkr_queue_sync, head);
 
-      mtx_unlock(&queue->mutex);
+      mtx_unlock(&queue->sync_thread.mutex);
 
       VkResult result;
       if (sync->device_lost) {
@@ -221,26 +181,48 @@ vkr_queue_thread(void *arg)
                                     ns_per_sec * 3);
       }
 
-      mtx_lock(&queue->mutex);
+      mtx_lock(&queue->sync_thread.mutex);
 
       if (result == VK_TIMEOUT)
          continue;
 
       list_del(&sync->head);
 
-      if (vkr_renderer_flags & VKR_RENDERER_ASYNC_FENCE_CB) {
-         ctx->base.fence_retire(&ctx->base, sync->ring_idx, sync->fence_id);
-         vkr_device_free_queue_sync(queue->device, sync);
-      } else {
-         list_addtail(&sync->head, &queue->signaled_syncs);
-         write_eventfd(queue->eventfd, 1);
-      }
+      vkr_queue_sync_retire(queue, sync);
    }
-   mtx_unlock(&queue->mutex);
+   mtx_unlock(&queue->sync_thread.mutex);
 
    return 0;
 }
 
+static int
+vkr_queue_sync_thread_init(struct vkr_queue *queue)
+{
+   STATIC_ASSERT(thrd_success == 0);
+
+   int ret = mtx_init(&queue->sync_thread.mutex, mtx_plain);
+   if (ret != thrd_success)
+      return ret;
+
+   ret = cnd_init(&queue->sync_thread.cond);
+   if (ret != thrd_success)
+      goto fail_cnd_init;
+
+   list_inithead(&queue->sync_thread.syncs);
+
+   ret = thrd_create(&queue->sync_thread.thread, vkr_queue_thread, queue);
+   if (ret != thrd_success)
+      goto fail_thrd_create;
+
+   return 0;
+
+fail_thrd_create:
+   mtx_destroy(&queue->sync_thread.mutex);
+fail_cnd_init:
+   cnd_destroy(&queue->sync_thread.cond);
+   return ret;
+}
+
 struct vkr_queue *
 vkr_queue_create(struct vkr_context *ctx,
                  struct vkr_device *dev,
@@ -249,11 +231,8 @@ vkr_queue_create(struct vkr_context *ctx,
                  uint32_t index,
                  VkQueue handle)
 {
-   struct vkr_queue *queue;
-   int ret;
-
    /* id is set to 0 until vkr_queue_assign_object_id */
-   queue = vkr_object_alloc(sizeof(*queue), VK_OBJECT_TYPE_QUEUE, 0);
+   struct vkr_queue *queue = vkr_object_alloc(sizeof(*queue), VK_OBJECT_TYPE_QUEUE, 0);
    if (!queue)
       return NULL;
 
@@ -265,38 +244,48 @@ vkr_queue_create(struct vkr_context *ctx,
    queue->family = family;
    queue->index = index;
 
-   list_inithead(&queue->pending_syncs);
-   list_inithead(&queue->signaled_syncs);
-
-   ret = mtx_init(&queue->mutex, mtx_plain);
-   if (ret != thrd_success) {
-      free(queue);
-      return NULL;
-   }
-   ret = cnd_init(&queue->cond);
-   if (ret != thrd_success) {
-      mtx_destroy(&queue->mutex);
+   if (mtx_init(&queue->vk_mutex, mtx_plain)) {
       free(queue);
       return NULL;
    }
 
-   if (vkr_renderer_flags & VKR_RENDERER_THREAD_SYNC) {
-      ret = thrd_create(&queue->thread, vkr_queue_thread, queue);
-      if (ret != thrd_success) {
-         mtx_destroy(&queue->mutex);
-         cnd_destroy(&queue->cond);
-         free(queue);
-         return NULL;
-      }
-      queue->eventfd = ctx->fence_eventfd;
+   if (vkr_queue_sync_thread_init(queue)) {
+      mtx_destroy(&queue->vk_mutex);
+      free(queue);
+      return NULL;
    }
 
-   list_inithead(&queue->busy_head);
    list_inithead(&queue->base.track_head);
 
    return queue;
 }
 
+static bool MUST_CHECK
+vkr_queue_assign_ring_idx(struct vkr_context *ctx,
+                          struct vkr_queue *queue,
+                          const VkDeviceQueueTimelineInfoMESA *timeline_info)
+{
+   if (unlikely(!timeline_info)) {
+      vkr_log("missing VkDeviceQueueTimelineInfoMESA");
+      return false;
+   }
+
+   const uint32_t ring_idx = timeline_info->ringIdx;
+   if (unlikely(!ring_idx || ring_idx >= ARRAY_SIZE(ctx->sync_queues))) {
+      vkr_log("invalid ring_idx %u", ring_idx);
+      return false;
+   }
+
+   if (unlikely(ctx->sync_queues[ring_idx])) {
+      vkr_log("sync_queue is already bound to ring_idx %u", ring_idx);
+      return false;
+   }
+
+   queue->ring_idx = ring_idx;
+   ctx->sync_queues[ring_idx] = queue;
+   return true;
+}
+
 static void
 vkr_queue_assign_object_id(struct vkr_context *ctx,
                            struct vkr_queue *queue,
@@ -304,7 +293,7 @@ vkr_queue_assign_object_id(struct vkr_context *ctx,
 {
    if (queue->base.id) {
       if (queue->base.id != id)
-         vkr_cs_decoder_set_fatal(&ctx->decoder);
+         vkr_context_set_fatal(ctx);
       return;
    }
    if (!vkr_context_validate_object_id(ctx, id))
@@ -321,9 +310,7 @@ vkr_device_lookup_queue(struct vkr_device *dev,
                         uint32_t family,
                         uint32_t index)
 {
-   struct vkr_queue *queue;
-
-   LIST_FOR_EACH_ENTRY (queue, &dev->queues, base.track_head) {
+   list_for_each_entry (struct vkr_queue, queue, &dev->queues, base.track_head) {
       if (queue->flags == flags && queue->family == family && queue->index == index)
          return queue;
    }
@@ -331,26 +318,6 @@ vkr_device_lookup_queue(struct vkr_device *dev,
    return NULL;
 }
 
-static void
-vkr_dispatch_vkGetDeviceQueue(struct vn_dispatch_context *dispatch,
-                              struct vn_command_vkGetDeviceQueue *args)
-{
-   struct vkr_context *ctx = dispatch->data;
-
-   struct vkr_device *dev = vkr_device_from_handle(args->device);
-
-   struct vkr_queue *queue = vkr_device_lookup_queue(
-      dev, 0 /* flags */, args->queueFamilyIndex, args->queueIndex);
-   if (!queue) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
-      return;
-   }
-
-   const vkr_object_id id =
-      vkr_cs_handle_load_id((const void **)args->pQueue, VK_OBJECT_TYPE_QUEUE);
-   vkr_queue_assign_object_id(ctx, queue, id);
-}
-
 static void
 vkr_dispatch_vkGetDeviceQueue2(struct vn_dispatch_context *dispatch,
                                struct vn_command_vkGetDeviceQueue2 *args)
@@ -363,28 +330,24 @@ vkr_dispatch_vkGetDeviceQueue2(struct vn_dispatch_context *dispatch,
                                                      args->pQueueInfo->queueFamilyIndex,
                                                      args->pQueueInfo->queueIndex);
    if (!queue) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
+      return;
+   }
+
+   /* Venus driver implementation is required to retrieve device queue only once to avoid
+    * overriding vkr_queue object id assignment.
+    */
+   if (queue->base.id) {
+      vkr_log("invalid to reinitialize vkr_queue");
+      vkr_context_set_fatal(ctx);
       return;
    }
 
    const VkDeviceQueueTimelineInfoMESA *timeline_info = vkr_find_struct(
       args->pQueueInfo->pNext, VK_STRUCTURE_TYPE_DEVICE_QUEUE_TIMELINE_INFO_MESA);
-   if (timeline_info) {
-      if (timeline_info->ringIdx == 0 ||
-          timeline_info->ringIdx >= ARRAY_SIZE(ctx->sync_queues)) {
-         vkr_log("invalid ring_idx %d", timeline_info->ringIdx);
-         vkr_cs_decoder_set_fatal(&ctx->decoder);
-         return;
-      }
-
-      if (ctx->sync_queues[timeline_info->ringIdx]) {
-         vkr_log("sync_queue %d already bound", timeline_info->ringIdx);
-         vkr_cs_decoder_set_fatal(&ctx->decoder);
-         return;
-      }
-
-      queue->ring_idx = timeline_info->ringIdx;
-      ctx->sync_queues[timeline_info->ringIdx] = queue;
+   if (!vkr_queue_assign_ring_idx(ctx, queue, timeline_info)) {
+      vkr_context_set_fatal(ctx);
+      return;
    }
 
    const vkr_object_id id =
@@ -392,28 +355,46 @@ vkr_dispatch_vkGetDeviceQueue2(struct vn_dispatch_context *dispatch,
    vkr_queue_assign_object_id(ctx, queue, id);
 }
 
+static void
+vkr_dispatch_vkGetDeviceQueue(struct vn_dispatch_context *dispatch,
+                              UNUSED struct vn_command_vkGetDeviceQueue *args)
+{
+   /* Must use vkGetDeviceQueue2 for proper device queue initialization. */
+   struct vkr_context *ctx = dispatch->data;
+   vkr_context_set_fatal(ctx);
+   return;
+}
+
 static void
 vkr_dispatch_vkQueueSubmit(UNUSED struct vn_dispatch_context *dispatch,
                            struct vn_command_vkQueueSubmit *args)
 {
+   TRACE_FUNC();
    struct vkr_queue *queue = vkr_queue_from_handle(args->queue);
    struct vn_device_proc_table *vk = &queue->device->proc_table;
 
    vn_replace_vkQueueSubmit_args_handle(args);
+
+   mtx_lock(&queue->vk_mutex);
    args->ret =
       vk->QueueSubmit(args->queue, args->submitCount, args->pSubmits, args->fence);
+   mtx_unlock(&queue->vk_mutex);
 }
 
 static void
 vkr_dispatch_vkQueueBindSparse(UNUSED struct vn_dispatch_context *dispatch,
                                struct vn_command_vkQueueBindSparse *args)
 {
+   TRACE_FUNC();
    struct vkr_queue *queue = vkr_queue_from_handle(args->queue);
    struct vn_device_proc_table *vk = &queue->device->proc_table;
 
    vn_replace_vkQueueBindSparse_args_handle(args);
+
+   mtx_lock(&queue->vk_mutex);
    args->ret =
       vk->QueueBindSparse(args->queue, args->bindInfoCount, args->pBindInfo, args->fence);
+   mtx_unlock(&queue->vk_mutex);
 }
 
 static void
@@ -422,19 +403,23 @@ vkr_dispatch_vkQueueWaitIdle(struct vn_dispatch_context *dispatch,
 {
    struct vkr_context *ctx = dispatch->data;
    /* no blocking call */
-   vkr_cs_decoder_set_fatal(&ctx->decoder);
+   vkr_context_set_fatal(ctx);
 }
 
 static void
 vkr_dispatch_vkQueueSubmit2(UNUSED struct vn_dispatch_context *dispatch,
                             struct vn_command_vkQueueSubmit2 *args)
 {
+   TRACE_FUNC();
    struct vkr_queue *queue = vkr_queue_from_handle(args->queue);
    struct vn_device_proc_table *vk = &queue->device->proc_table;
 
    vn_replace_vkQueueSubmit2_args_handle(args);
+
+   mtx_lock(&queue->vk_mutex);
    args->ret =
       vk->QueueSubmit2(args->queue, args->submitCount, args->pSubmits, args->fence);
+   mtx_unlock(&queue->vk_mutex);
 }
 
 static void
@@ -474,32 +459,27 @@ vkr_dispatch_vkGetFenceStatus(UNUSED struct vn_dispatch_context *dispatch,
 }
 
 static void
-vkr_dispatch_vkWaitForFences(struct vn_dispatch_context *dispatch,
+vkr_dispatch_vkWaitForFences(UNUSED struct vn_dispatch_context *dispatch,
                              struct vn_command_vkWaitForFences *args)
 {
-   struct vkr_context *ctx = dispatch->data;
    struct vkr_device *dev = vkr_device_from_handle(args->device);
    struct vn_device_proc_table *vk = &dev->proc_table;
 
    vn_replace_vkWaitForFences_args_handle(args);
    args->ret = vk->WaitForFences(args->device, args->fenceCount, args->pFences,
                                  args->waitAll, args->timeout);
-
-   if (args->ret == VK_ERROR_DEVICE_LOST)
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
 }
 
 static void
-vkr_dispatch_vkResetFenceResource100000MESA(
-   struct vn_dispatch_context *dispatch,
-   struct vn_command_vkResetFenceResource100000MESA *args)
+vkr_dispatch_vkResetFenceResourceMESA(struct vn_dispatch_context *dispatch,
+                                      struct vn_command_vkResetFenceResourceMESA *args)
 {
    struct vkr_context *ctx = dispatch->data;
    struct vkr_device *dev = vkr_device_from_handle(args->device);
    struct vn_device_proc_table *vk = &dev->proc_table;
    int fd = -1;
 
-   vn_replace_vkResetFenceResource100000MESA_args_handle(args);
+   vn_replace_vkResetFenceResourceMESA_args_handle(args);
 
    const VkFenceGetFdInfoKHR info = {
       .sType = VK_STRUCTURE_TYPE_FENCE_GET_FD_INFO_KHR,
@@ -508,7 +488,7 @@ vkr_dispatch_vkResetFenceResource100000MESA(
    };
    VkResult result = vk->GetFenceFdKHR(args->device, &info, &fd);
    if (result != VK_SUCCESS) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
@@ -542,18 +522,14 @@ vkr_dispatch_vkGetSemaphoreCounterValue(UNUSED struct vn_dispatch_context *dispa
 }
 
 static void
-vkr_dispatch_vkWaitSemaphores(struct vn_dispatch_context *dispatch,
+vkr_dispatch_vkWaitSemaphores(UNUSED struct vn_dispatch_context *dispatch,
                               struct vn_command_vkWaitSemaphores *args)
 {
-   struct vkr_context *ctx = dispatch->data;
    struct vkr_device *dev = vkr_device_from_handle(args->device);
    struct vn_device_proc_table *vk = &dev->proc_table;
 
    vn_replace_vkWaitSemaphores_args_handle(args);
    args->ret = vk->WaitSemaphores(args->device, args->pWaitInfo, args->timeout);
-
-   if (args->ret == VK_ERROR_DEVICE_LOST)
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
 }
 
 static void
@@ -568,16 +544,16 @@ vkr_dispatch_vkSignalSemaphore(UNUSED struct vn_dispatch_context *dispatch,
 }
 
 static void
-vkr_dispatch_vkWaitSemaphoreResource100000MESA(
+vkr_dispatch_vkWaitSemaphoreResourceMESA(
    struct vn_dispatch_context *dispatch,
-   struct vn_command_vkWaitSemaphoreResource100000MESA *args)
+   struct vn_command_vkWaitSemaphoreResourceMESA *args)
 {
    struct vkr_context *ctx = dispatch->data;
    struct vkr_device *dev = vkr_device_from_handle(args->device);
    struct vn_device_proc_table *vk = &dev->proc_table;
    int fd = -1;
 
-   vn_replace_vkWaitSemaphoreResource100000MESA_args_handle(args);
+   vn_replace_vkWaitSemaphoreResourceMESA_args_handle(args);
 
    const VkSemaphoreGetFdInfoKHR info = {
       .sType = VK_STRUCTURE_TYPE_SEMAPHORE_GET_FD_INFO_KHR,
@@ -586,7 +562,7 @@ vkr_dispatch_vkWaitSemaphoreResource100000MESA(
    };
    VkResult result = vk->GetSemaphoreFdKHR(args->device, &info, &fd);
    if (result != VK_SUCCESS) {
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
       return;
    }
 
@@ -595,18 +571,17 @@ vkr_dispatch_vkWaitSemaphoreResource100000MESA(
 }
 
 static void
-vkr_dispatch_vkImportSemaphoreResource100000MESA(
+vkr_dispatch_vkImportSemaphoreResourceMESA(
    struct vn_dispatch_context *dispatch,
-   struct vn_command_vkImportSemaphoreResource100000MESA *args)
+   struct vn_command_vkImportSemaphoreResourceMESA *args)
 {
    struct vkr_context *ctx = dispatch->data;
    struct vkr_device *dev = vkr_device_from_handle(args->device);
    struct vn_device_proc_table *vk = &dev->proc_table;
 
-   vn_replace_vkImportSemaphoreResource100000MESA_args_handle(args);
+   vn_replace_vkImportSemaphoreResourceMESA_args_handle(args);
 
-   const VkImportSemaphoreResourceInfo100000MESA *res_info =
-      args->pImportSemaphoreResourceInfo;
+   const VkImportSemaphoreResourceInfoMESA *res_info = args->pImportSemaphoreResourceInfo;
 
    /* resourceId 0 is for importing a signaled payload to sync_fd fence */
    assert(!res_info->resourceId);
@@ -619,7 +594,7 @@ vkr_dispatch_vkImportSemaphoreResource100000MESA(
       .fd = -1,
    };
    if (vk->ImportSemaphoreFdKHR(args->device, &import_info) != VK_SUCCESS)
-      vkr_cs_decoder_set_fatal(&ctx->decoder);
+      vkr_context_set_fatal(ctx);
 }
 
 static void
@@ -695,8 +670,7 @@ vkr_context_init_fence_dispatch(struct vkr_context *ctx)
    dispatch->dispatch_vkGetFenceStatus = vkr_dispatch_vkGetFenceStatus;
    dispatch->dispatch_vkWaitForFences = vkr_dispatch_vkWaitForFences;
 
-   dispatch->dispatch_vkResetFenceResource100000MESA =
-      vkr_dispatch_vkResetFenceResource100000MESA;
+   dispatch->dispatch_vkResetFenceResourceMESA = vkr_dispatch_vkResetFenceResourceMESA;
 }
 
 void
@@ -711,10 +685,10 @@ vkr_context_init_semaphore_dispatch(struct vkr_context *ctx)
    dispatch->dispatch_vkWaitSemaphores = vkr_dispatch_vkWaitSemaphores;
    dispatch->dispatch_vkSignalSemaphore = vkr_dispatch_vkSignalSemaphore;
 
-   dispatch->dispatch_vkWaitSemaphoreResource100000MESA =
-      vkr_dispatch_vkWaitSemaphoreResource100000MESA;
-   dispatch->dispatch_vkImportSemaphoreResource100000MESA =
-      vkr_dispatch_vkImportSemaphoreResource100000MESA;
+   dispatch->dispatch_vkWaitSemaphoreResourceMESA =
+      vkr_dispatch_vkWaitSemaphoreResourceMESA;
+   dispatch->dispatch_vkImportSemaphoreResourceMESA =
+      vkr_dispatch_vkImportSemaphoreResourceMESA;
 }
 
 void
