@@ -6,6 +6,7 @@
 #include "vkr_ring.h"
 
 #include <stdio.h>
+#include <sys/resource.h>
 #include <time.h>
 
 #include "venus-protocol/vn_protocol_renderer_dispatches.h"
@@ -128,7 +129,9 @@ vkr_ring_create(const struct vkr_ring_layout *layout,
    if (!ring->cmd)
       goto err_cmd_malloc;
 
-   vkr_cs_decoder_init(&ring->decoder, &ctx->cs_fatal_error, ctx->object_table);
+   if (vkr_cs_decoder_init(&ring->decoder, ctx))
+      goto err_cs_decoder_init;
+
    if (vkr_cs_encoder_init(&ring->encoder, &ctx->cs_fatal_error))
       goto err_cs_encoder_init;
 
@@ -149,6 +152,8 @@ err_cond_init:
 err_mtx_init:
    vkr_cs_encoder_fini(&ring->encoder);
 err_cs_encoder_init:
+   vkr_cs_decoder_fini(&ring->decoder);
+err_cs_decoder_init:
    free(ring->cmd);
 err_cmd_malloc:
 err_init_control:
@@ -212,7 +217,7 @@ vkr_ring_submit_cmd(struct vkr_ring *ring,
       return false;
    }
 
-   vkr_cs_decoder_set_stream(dec, buffer, size);
+   vkr_cs_decoder_set_buffer_stream(dec, buffer, size);
 
    while (vkr_cs_decoder_has_command(dec)) {
       vn_dispatch_command(&ring->dispatch);
@@ -242,6 +247,15 @@ vkr_ring_thread(void *arg)
 
    snprintf(thread_name, ARRAY_SIZE(thread_name), "vkr-ring-%d", ctx->ctx_id);
    u_thread_setname(thread_name);
+   if (ring->prio_valid && setpriority(PRIO_PROCESS, 0, ring->prio)) {
+#ifdef DEBUG
+      /* Currently venus doesn't forward the CAP_SYS_NICE request upon forking, so
+       * requesting a high priority outside of the normal range would be at the best
+       * effort. Thus only enable logging on debug build.
+       */
+      vkr_log("failed to set ring thread priority to %d (%d)", ring->prio, errno);
+#endif
+   }
 
    uint64_t last_submit = vkr_ring_now();
    uint32_t relax_iter = 0;
@@ -275,6 +289,8 @@ vkr_ring_thread(void *arg)
       const uint32_t cmd_size = vkr_ring_load_tail(ring) - ring->buffer.cur;
       if (cmd_size) {
          if (cmd_size > ring->buffer.size) {
+            vkr_log("%s: cmd_size(%u) > ring->buffer.size(%u)", __func__, cmd_size,
+                    ring->buffer.size);
             ret = -EINVAL;
             break;
          }
@@ -290,12 +306,30 @@ vkr_ring_thread(void *arg)
          last_submit = vkr_ring_now();
          relax_iter = 0;
       } else {
+         /* Get the active wait_ring seqno first to ensure ordering. */
+         uint32_t wait_ring_seqno = 0;
+         if (vkr_context_get_wait_ring_seqno(ctx, ring->id, &wait_ring_seqno)) {
+            /* Error out if the latest ring cmd is unable to signal the virtqueue that is
+             * currently waiting for this ring. This happens when the driver emits invalid
+             * asynchronous ring wait cmds.
+             */
+            const uint32_t ring_tail = vkr_ring_load_tail(ring);
+            if (unlikely(!vkr_seqno_ge(ring_tail, wait_ring_seqno))) {
+               vkr_log("%s: ring seqno(%u) unable to reach wait seqno(%u)", __func__,
+                       ring_tail, wait_ring_seqno);
+               ret = -EINVAL;
+               break;
+            }
+         }
+
          vkr_ring_relax(&relax_iter);
       }
    }
 
-   if (ret < 0)
+   if (ret < 0) {
       vkr_ring_set_status_bits(ring, VK_RING_STATUS_FATAL_BIT_MESA);
+      vkr_context_on_ring_fatal(ctx);
+   }
 
    return ret;
 }
