@@ -28,10 +28,12 @@
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
+#include <unistd.h>
 
-#include "virglrenderer_hw.h"
 #include "virgl_resource.h"
 
+#include "util/list.h"
+
 struct vrend_transfer_info;
 struct pipe_resource;
 
@@ -46,7 +48,7 @@ struct virgl_context_blob {
 
    uint32_t map_info;
 
-   struct virgl_resource_opaque_fd_metadata opaque_fd_metadata;
+   struct virgl_resource_vulkan_info vulkan_info;
 };
 
 struct virgl_context;
@@ -62,7 +64,9 @@ typedef void (*virgl_context_fence_retire)(struct virgl_context *ctx,
 struct virgl_context {
    uint32_t ctx_id;
 
-   enum virgl_renderer_capset capset_id;
+   int in_fence_fd;
+
+   uint32_t capset_id;
 
    /*
     * Each fence goes through submitted, signaled, and retired.  This callback
@@ -74,6 +78,8 @@ struct virgl_context {
     */
    virgl_context_fence_retire fence_retire;
 
+   bool supports_fence_sharing;
+
    void (*destroy)(struct virgl_context *ctx);
 
    void (*attach_resource)(struct virgl_context *ctx,
@@ -117,11 +123,14 @@ struct virgl_context {
    /* retire signaled fences of all queues */
    void (*retire_fences)(struct virgl_context *ctx);
 
-   /* submit a fence to the queue identified by queue_id */
+   /* submit a fence to the queue identified by ring_idx */
    int (*submit_fence)(struct virgl_context *ctx,
                        uint32_t flags,
                        uint32_t ring_idx,
                        uint64_t fence_id);
+
+   /* For DRM native contexts, return the device fd: */
+   int (*get_device_fd)(struct virgl_context *ctx);
 };
 
 struct virgl_context_foreach_args {
@@ -150,4 +159,6 @@ virgl_context_lookup(uint32_t ctx_id);
 void
 virgl_context_foreach(const struct virgl_context_foreach_args *args);
 
+int virgl_context_take_in_fence_fd(struct virgl_context *ctx);
+
 #endif /* VIRGL_CONTEXT_H */
