@@ -42,16 +42,16 @@
 #include "util/u_math.h"
 #include "vkr_allocator.h"
 #include "drm_renderer.h"
-#include "vrend_renderer.h"
 #include "proxy/proxy_renderer.h"
-#include "vrend_winsys.h"
+#include "vrend/vrend_renderer.h"
+#include "vrend/vrend_winsys.h"
 
 #ifndef WIN32
 #include "util/libsync.h"
 #endif
 
 #include "virglrenderer.h"
-#include "virglrenderer_hw.h"
+#include "virtgpu_drm.h"
 
 #include "virgl_context.h"
 #include "virgl_fence.h"
@@ -85,14 +85,18 @@ static int virgl_renderer_resource_create_internal(struct virgl_renderer_resourc
    struct virgl_resource *res;
    struct pipe_resource *pipe_res;
    struct vrend_renderer_resource_create_args vrend_args =  { 0 };
+   uint32_t map_info;
 
-   if (!state.vrend_initialized)
+   if (!state.vrend_initialized && !state.drm_initialized)
       return EINVAL;
 
    /* do not accept handle 0 */
    if (args->handle == 0)
       return EINVAL;
 
+   if (virgl_resource_lookup(args->handle))
+      return -EINVAL;
+
    vrend_args.target = args->target;
    vrend_args.format = args->format;
    vrend_args.bind = args->bind;
@@ -108,13 +112,12 @@ static int virgl_renderer_resource_create_internal(struct virgl_renderer_resourc
    if (!pipe_res)
       return EINVAL;
 
+   map_info = vrend_renderer_resource_get_map_info(pipe_res);
    res = virgl_resource_create_from_pipe(args->handle, pipe_res, iov, num_iovs);
-   if (!res) {
-      vrend_renderer_resource_destroy((struct vrend_resource *)pipe_res);
+   if (!res)
       return -ENOMEM;
-   }
 
-   res->map_info = vrend_renderer_resource_get_map_info(pipe_res);
+   res->map_info = map_info;
 
    return 0;
 }
@@ -176,16 +179,16 @@ void virgl_renderer_fill_caps(uint32_t set, uint32_t version,
                               void *caps)
 {
    switch (set) {
-   case VIRGL_RENDERER_CAPSET_VIRGL:
-   case VIRGL_RENDERER_CAPSET_VIRGL2:
+   case VIRTGPU_DRM_CAPSET_VIRGL:
+   case VIRTGPU_DRM_CAPSET_VIRGL2:
       if (state.vrend_initialized)
          vrend_renderer_fill_caps(set, version, (union virgl_caps *)caps);
       break;
-   case VIRGL_RENDERER_CAPSET_VENUS:
+   case VIRTGPU_DRM_CAPSET_VENUS:
       if (state.proxy_initialized)
          proxy_get_capset(set, caps);
       break;
-   case VIRGL_RENDERER_CAPSET_DRM:
+   case VIRTGPU_DRM_CAPSET_DRM:
       if (state.drm_initialized)
          drm_renderer_capset(caps);
       break;
@@ -209,8 +212,7 @@ int virgl_renderer_context_create_with_flags(uint32_t ctx_id,
                                              uint32_t nlen,
                                              const char *name)
 {
-   const enum virgl_renderer_capset capset_id =
-      ctx_flags & VIRGL_RENDERER_CONTEXT_FLAG_CAPSET_ID_MASK;
+   uint32_t capset_id = ctx_flags & VIRGL_RENDERER_CONTEXT_FLAG_CAPSET_ID_MASK;
    struct virgl_context *ctx;
    int ret;
 
@@ -230,18 +232,18 @@ int virgl_renderer_context_create_with_flags(uint32_t ctx_id,
    }
 
    switch (capset_id) {
-   case VIRGL_RENDERER_CAPSET_VIRGL:
-   case VIRGL_RENDERER_CAPSET_VIRGL2:
+   case VIRTGPU_DRM_CAPSET_VIRGL:
+   case VIRTGPU_DRM_CAPSET_VIRGL2:
       if (!state.vrend_initialized)
          return EINVAL;
       ctx = vrend_renderer_context_create(ctx_id, nlen, name);
       break;
-   case VIRGL_RENDERER_CAPSET_VENUS:
+   case VIRTGPU_DRM_CAPSET_VENUS:
       if (!state.proxy_initialized)
          return EINVAL;
       ctx = proxy_context_create(ctx_id, ctx_flags, nlen, name);
       break;
-   case VIRGL_RENDERER_CAPSET_DRM:
+   case VIRTGPU_DRM_CAPSET_DRM:
       if (!state.drm_initialized)
          return EINVAL;
       ctx = drm_renderer_create(nlen, name);
@@ -270,7 +272,7 @@ int virgl_renderer_context_create_with_flags(uint32_t ctx_id,
 int virgl_renderer_context_create(uint32_t handle, uint32_t nlen, const char *name)
 {
    return virgl_renderer_context_create_with_flags(handle,
-                                                   VIRGL_RENDERER_CAPSET_VIRGL2,
+                                                   VIRTGPU_DRM_CAPSET_VIRGL2,
                                                    nlen,
                                                    name);
 }
@@ -293,7 +295,10 @@ int virgl_renderer_submit_cmd(void *buffer,
    if (ndw < 0 || (unsigned)ndw > UINT32_MAX / sizeof(uint32_t))
       return EINVAL;
 
-   return ctx->submit_cmd(ctx, buffer, ndw * sizeof(uint32_t));
+   if (((uintptr_t)buffer & 3) != 0)
+      return EFAULT;
+
+   return ctx->submit_cmd(ctx, buffer, (uint32_t)ndw * sizeof(uint32_t));
 }
 
 int virgl_renderer_transfer_write_iov(uint32_t handle,
@@ -429,6 +434,7 @@ int virgl_renderer_context_create_fence(uint32_t ctx_id,
 
 void virgl_renderer_context_poll(uint32_t ctx_id)
 {
+   TRACE_FUNC();
    struct virgl_context *ctx = virgl_context_lookup(ctx_id);
    if (!ctx)
       return;
@@ -555,15 +561,15 @@ void virgl_renderer_get_cap_set(uint32_t cap_set, uint32_t *max_ver,
 
    /* this may be called before virgl_renderer_init */
    switch (cap_set) {
-   case VIRGL_RENDERER_CAPSET_VIRGL:
-   case VIRGL_RENDERER_CAPSET_VIRGL2:
+   case VIRTGPU_DRM_CAPSET_VIRGL:
+   case VIRTGPU_DRM_CAPSET_VIRGL2:
       vrend_renderer_get_cap_set(cap_set, max_ver, max_size);
       break;
-   case VIRGL_RENDERER_CAPSET_VENUS:
+   case VIRTGPU_DRM_CAPSET_VENUS:
       *max_ver = 0;
       *max_size = proxy_get_capset(cap_set, NULL);
       break;
-   case VIRGL_RENDERER_CAPSET_DRM:
+   case VIRTGPU_DRM_CAPSET_DRM:
       *max_ver = 0;
       *max_size = drm_renderer_capset(NULL);
       break;
@@ -729,8 +735,8 @@ virgl_context_foreach_retire_fences(struct virgl_context *ctx,
                                     UNUSED void* data)
 {
    /* vrend contexts are polled explicitly by the caller */
-   if (ctx->capset_id != VIRGL_RENDERER_CAPSET_VIRGL &&
-       ctx->capset_id != VIRGL_RENDERER_CAPSET_VIRGL2 &&
+   if (ctx->capset_id != VIRTGPU_DRM_CAPSET_VIRGL &&
+       ctx->capset_id != VIRTGPU_DRM_CAPSET_VIRGL2 &&
        !(state.flags & VIRGL_RENDERER_ASYNC_FENCE_CB))
    {
       assert(ctx->retire_fences);
@@ -796,14 +802,18 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
 
    if (state.client_initialized && (state.cookie != cookie ||
                                     state.flags != flags ||
-                                    state.cbs != cbs))
+                                    state.cbs != cbs)) {
+      virgl_error("renderer already initialized");
       return -EBUSY;
+   }
 
    if (!state.client_initialized) {
       if (!cbs ||
           cbs->version < 1 ||
-          cbs->version > VIRGL_RENDERER_CALLBACKS_VERSION)
+          cbs->version > VIRGL_RENDERER_CALLBACKS_VERSION) {
+         virgl_error("invalid renderer callbacks");
          return -1;
+      }
 
       state.cookie = cookie;
       state.flags = flags;
@@ -817,15 +827,19 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
          vrend_renderer_get_pipe_callbacks();
 
       ret = virgl_resource_table_init(pipe_cbs);
-      if (ret)
+      if (ret) {
+         virgl_error("failed to initialize virgl resources");
          goto fail;
+      }
       state.resource_initialized = true;
    }
 
    if (!state.context_initialized) {
       ret = virgl_context_table_init();
-      if (ret)
+      if (ret) {
+         virgl_error("failed to initialize virgl context");
          goto fail;
+      }
       state.context_initialized = true;
    }
 
@@ -842,6 +856,7 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
       if (ret) {
          if (drm_fd >= 0)
             close(drm_fd);
+         virgl_error("failed to initialize vrend winsys");
          goto fail;
       }
       state.winsys_initialized = true;
@@ -853,6 +868,7 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
 
       if (!cbs->create_gl_context || !cbs->destroy_gl_context ||
           !cbs->make_current) {
+         virgl_error("invalid renderer gl callbacks");
          ret = EINVAL;
          goto fail;
       }
@@ -860,12 +876,14 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
       egl_display = state.cbs->get_egl_display(cookie);
 
       if (!egl_display) {
+         virgl_error("failed to get egl display");
          ret = -1;
          goto fail;
       }
       ret = vrend_winsys_init_external(egl_display);
 
       if (ret) {
+         virgl_error("failed to initialize vrend winsys");
          ret = -1;
          goto fail;
       }
@@ -877,6 +895,7 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
       uint32_t renderer_flags = 0;
 
       if (!cookie || !cbs) {
+         virgl_error("invalid renderer vrend callbacks");
          ret = -1;
          goto fail;
       }
@@ -893,17 +912,23 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
          renderer_flags |= VREND_D3D11_SHARE_TEXTURE;
       if (flags & VIRGL_RENDERER_COMPAT_PROFILE)
          renderer_flags |= VREND_USE_COMPAT_CONTEXT;
+      if (flags & VIRGL_RENDERER_USE_GLES)
+         renderer_flags |= VREND_USE_GLES;
 
       ret = vrend_renderer_init(&vrend_cbs, renderer_flags);
-      if (ret)
+      if (ret) {
+         virgl_error("failed to initialize vrend renderer");
          goto fail;
+      }
       state.vrend_initialized = true;
    }
 
    if (!state.proxy_initialized && (flags & VIRGL_RENDERER_RENDER_SERVER)) {
       ret = proxy_renderer_init(&proxy_cbs, flags | VIRGL_RENDERER_NO_VIRGL);
-      if (ret)
+      if (ret) {
+         virgl_error("failed to initialize venus renderer");
          goto fail;
+      }
       state.proxy_initialized = true;
    }
 
@@ -915,15 +940,19 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
          drm_fd = cbs->get_drm_fd(cookie);
 
       ret = drm_renderer_init(drm_fd);
-      if (ret)
+      if (ret) {
+         virgl_error("failed to initialize drm renderer");
          goto fail;
+      }
       state.drm_initialized = true;
    }
 
    if (!state.fence_initialized) {
       ret = virgl_fence_table_init();
-      if (ret)
+      if (ret) {
+         virgl_error("failed to initialize fence table");
          goto fail;
+      }
       state.fence_initialized = true;
    }
 
@@ -1176,19 +1205,15 @@ int virgl_renderer_resource_create_blob(const struct virgl_renderer_resource_cre
                                           args->iovecs,
                                           args->num_iovs,
                                           &blob.vulkan_info);
-      if (!res) {
-         close(blob.u.fd);
+      if (!res)
          return -ENOMEM;
-      }
    } else {
       res = virgl_resource_create_from_pipe(args->res_handle,
                                             blob.u.pipe_resource,
                                             args->iovecs,
                                             args->num_iovs);
-      if (!res) {
-         vrend_renderer_resource_destroy((struct vrend_resource *)blob.u.pipe_resource);
+      if (!res)
          return -ENOMEM;
-      }
    }
 
    res->map_info = blob.map_info;
@@ -1212,10 +1237,18 @@ int virgl_renderer_resource_map(uint32_t res_handle, void **out_map, uint64_t *o
       if (!ret)
          res->map_size = map_size;
    } else {
-      switch (res->fd_type) {
+      enum virgl_resource_fd_type fd_type = res->fd_type;
+      enum virgl_resource_fd_type export_fd_type = res->fd_type;
+      int fd = res->fd;
+
+      /* Create a transient dmabuf. */
+      if (fd_type == VIRGL_RESOURCE_OPAQUE_HANDLE)
+         export_fd_type = virgl_resource_export_fd(res, &fd);
+
+      switch (export_fd_type) {
       case VIRGL_RESOURCE_FD_DMABUF:
       case VIRGL_RESOURCE_FD_SHM:
-         map = mmap(NULL, res->map_size, PROT_WRITE | PROT_READ, MAP_SHARED, res->fd, 0);
+         map = mmap(NULL, res->map_size, PROT_WRITE | PROT_READ, MAP_SHARED, fd, 0);
          map_size = res->map_size;
          break;
       case VIRGL_RESOURCE_FD_OPAQUE:
@@ -1228,6 +1261,9 @@ int virgl_renderer_resource_map(uint32_t res_handle, void **out_map, uint64_t *o
           */
          break;
       }
+
+      if (export_fd_type != fd_type)
+         close(fd);
    }
 
    if (!map || map == MAP_FAILED)
@@ -1253,12 +1289,12 @@ int virgl_renderer_resource_unmap(uint32_t res_handle)
       switch (res->fd_type) {
       case VIRGL_RESOURCE_FD_DMABUF:
       case VIRGL_RESOURCE_FD_SHM:
+      case VIRGL_RESOURCE_OPAQUE_HANDLE:
          ret = munmap(res->mapped, res->map_size);
          break;
       case VIRGL_RESOURCE_FD_OPAQUE:
          ret = vkr_allocator_resource_unmap(res);
          break;
-      case VIRGL_RESOURCE_OPAQUE_HANDLE:
       case VIRGL_RESOURCE_FD_INVALID:
          /* Avoid a default case so that -Wswitch will tell us at compile time
           * if a new virgl resource type is added without being handled here.
@@ -1395,10 +1431,40 @@ int virgl_renderer_export_signalled_fence(void)
    return virgl_fence_get_last_signalled_fence_fd();
 }
 
+static int attach_in_fence_fd(struct virgl_context *ctx, int fence_fd)
+{
+   int ret = -EINVAL;
+
+#ifndef WIN32
+   ret = sync_accumulate("virglrenderer", &ctx->in_fence_fd, fence_fd);
+#endif
+   close(fence_fd);
+
+   return ret;
+}
+
+/* Special entrypoint for vtest, which has received a real fence fd,
+ * not a fence-id
+ */
+int virgl_renderer_attach_fence(int ctx_id, int fence_fd)
+{
+   TRACE_FUNC();
+   struct virgl_context *ctx = virgl_context_lookup(ctx_id);
+   if (!ctx)
+      return EINVAL;
+
+   return attach_in_fence_fd(ctx, fence_fd);
+}
+
+int virgl_renderer_get_fence_fd(uint64_t fence_id)
+{
+   return virgl_fence_get_fd(fence_id);
+}
+
 static int virgl_renderer_context_attach_in_fence(struct virgl_context *ctx,
                                                   uint64_t fence_id)
 {
-   int ret = -EINVAL;
+   int ret;
 
    /*
     * FD will be -1 in two cases:
@@ -1413,11 +1479,7 @@ static int virgl_renderer_context_attach_in_fence(struct virgl_context *ctx,
    if (fd < 0)
       return 0;
 
-#ifndef WIN32
-   ret = sync_accumulate("virglrenderer", &ctx->in_fence_fd, fd);
-#endif
-   close(fd);
-
+   ret = attach_in_fence_fd(ctx, fd);
    if (ret)
       virgl_error("%s: sync_accumulate failed for fence_id=%" PRIu64 " err=%d\n",
                   __func__, fence_id, ret);
@@ -1454,6 +1516,9 @@ int virgl_renderer_submit_cmd2(void *buffer,
    if (!ctx)
       return EINVAL;
 
+   if (((uintptr_t)buffer & 3) != 0)
+      return EFAULT;
+
    if (ndw < 0 || (unsigned)ndw > UINT32_MAX / sizeof(uint32_t))
       return EINVAL;
 
@@ -1463,5 +1528,17 @@ int virgl_renderer_submit_cmd2(void *buffer,
          return err;
    }
 
-   return ctx->submit_cmd(ctx, buffer, ndw * sizeof(uint32_t));
+   return ctx->submit_cmd(ctx, buffer, (uint32_t)ndw * sizeof(uint32_t));
+}
+
+int virgl_renderer_get_dev_fd(int ctx_id)
+{
+   struct virgl_context *ctx = virgl_context_lookup(ctx_id);
+   if (!ctx)
+      return -EINVAL;
+
+   if (!ctx->get_device_fd)
+      return -ENODEV;
+
+   return ctx->get_device_fd(ctx);
 }
