@@ -18,8 +18,8 @@
 
 #include "../testvirgl_encode.h"
 #include "virgl_protocol.h"
-#include "vrend_iov.h"
-#include "vrend_debug.h"
+#include "vrend/vrend_iov.h"
+#include "vrend/vrend_debug.h"
 
 int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size);
 
