@@ -27,7 +27,7 @@
 #include <errno.h>
 #include <virglrenderer.h>
 #include "virgl_hw.h"
-#include "vrend_iov.h"
+#include "vrend/vrend_iov.h"
 #include "util/u_formats.h"
 #include "testvirgl_encode.h"
 #include "virgl_protocol.h"
