@@ -29,7 +29,7 @@
 #include <virglrenderer.h>
 #include "pipe/p_defines.h"
 #include "virgl_hw.h"
-#include "vrend_iov.h"
+#include "vrend/vrend_iov.h"
 #include "virgl_protocol.h"
 #include "testvirgl_encode.h"
 
