@@ -55,6 +55,7 @@ struct virgl_resource {
     uint32_t handle;
     struct iovec *iovs;
     int niovs;
+    int stride;
 };
 
 
@@ -62,14 +63,14 @@ void testvirgl_init_simple_buffer(struct virgl_renderer_resource_create_args *re
 void testvirgl_init_simple_buffer_sized(struct virgl_renderer_resource_create_args *res, int handle, int width);
 void testvirgl_init_simple_1d_resource(struct virgl_renderer_resource_create_args *args, int handle);
 void testvirgl_init_simple_2d_resource(struct virgl_renderer_resource_create_args *res, int handle);
-int testvirgl_init_single_ctx(void);
+int testvirgl_init_single_ctx(uint32_t flags);
 void testvirgl_init_single_ctx_nr(void);
 void testvirgl_fini_single_ctx(void);
 
 uint32_t testvirgl_get_last_fence(void);
 void testvirgl_reset_fence(void);
 
-int testvirgl_init_ctx_cmdbuf(struct virgl_context *ctx);
+int testvirgl_init_ctx_cmdbuf(struct virgl_context *ctx, uint32_t flags);
 int testvirgl_ctx_send_cmdbuf(struct virgl_context *ctx);
 void testvirgl_fini_ctx_cmdbuf(struct virgl_context *ctx);
 
