@@ -33,6 +33,21 @@ struct virgl_surface {
    uint32_t handle;
 };
 
+struct virgl_shader_image {
+   uint32_t format;
+   uint32_t access;
+   uint32_t layer_offset;
+   uint32_t level_size;
+   uint32_t handle;
+};
+
+struct virgl_shader_buffer {
+   uint32_t offset;
+   uint32_t buf_len;
+   uint32_t handle;
+};
+
+
 static inline void virgl_encoder_write_dword(struct virgl_cmd_buf *state,
                                             uint32_t dword)
 {
@@ -117,12 +132,22 @@ int virgl_encoder_set_viewport_states(struct virgl_context *ctx,
 int virgl_encoder_draw_vbo(struct virgl_context *ctx,
                           const struct pipe_draw_info *info);
 
+int virgl_encoder_draw_vbo_indirect(struct virgl_context *ctx,
+                                const struct pipe_draw_info *info,
+                                uint32_t indirect_handle,
+                                uint32_t indirect_draw_count_handle);
 
 int virgl_encoder_create_surface(struct virgl_context *ctx,
                                 uint32_t handle,
                                 struct virgl_resource *res,
                                 const struct pipe_surface *templat);
 
+int virgl_encoder_clear_texture(struct virgl_context *ctx,
+                                uint32_t handle,
+                                uint32_t level,
+                                struct virgl_box box,
+                                const union pipe_color_union *color);
+
 int virgl_encoder_flush_frontbuffer(struct virgl_context *ctx,
                                    struct virgl_resource *res);
 
@@ -178,12 +203,26 @@ int virgl_encode_set_sampler_views(struct virgl_context *ctx,
                                   uint32_t num_views,
                                   struct virgl_sampler_view **views);
 
+int virgl_encode_set_shader_images(struct virgl_context *ctx,
+                                   uint32_t shader_type,
+                                   uint32_t start_slot,
+                                   uint32_t num_views,
+                                   const struct virgl_shader_image *images);
+
+int virgl_encode_set_shader_buffers(struct virgl_context *ctx,
+                                    uint32_t shader_type,
+                                    uint32_t start_slot,
+                                    uint32_t num_buffers,
+                                    const struct virgl_shader_buffer *buffers);
+
 int virgl_encode_bind_sampler_states(struct virgl_context *ctx,
                                     uint32_t shader_type,
                                     uint32_t start_slot,
                                     uint32_t num_handles,
                                     uint32_t *handles);
 
+int virgl_encode_simple_launch_grid(struct virgl_context *ctx, uint32_t grid[3]);
+
 int virgl_encoder_set_index_buffer(struct virgl_context *ctx,
                                   const struct pipe_index_buffer *ib);
 
@@ -201,6 +240,34 @@ int virgl_encoder_set_uniform_buffer(struct virgl_context *ctx,
                                      uint32_t offset,
                                      uint32_t length,
                                      struct virgl_resource *res);
+
+struct vrend_image_view {
+   uint32_t id;
+   int access;
+   int format;
+   uint32_t vformat;
+   union {
+      struct {
+          unsigned first_layer:16;     /**< first layer to use for array textures */
+          unsigned last_layer:16;      /**< last layer to use for array textures */
+          unsigned level:8;            /**< mipmap level to use */
+      } tex;
+      struct {
+          unsigned offset;   /**< offset in bytes */
+          unsigned size;     /**< size of the accessible sub-range in bytes */
+      } buf;
+   } u;
+   struct vrend_resource *texture;
+   uint32_t view_id;
+};
+
+
+
+int virgl_encoder_set_shader_images(struct virgl_context *ctx,
+                                    uint32_t shader_type,
+                                    uint32_t start_slot,
+                                    uint32_t num_images,
+                                    struct vrend_image_view *images, uint32_t handle);
 int virgl_encode_dsa_state(struct virgl_context *ctx,
                           uint32_t handle,
                           const struct pipe_depth_stencil_alpha_state *dsa_state);
@@ -240,7 +307,7 @@ int virgl_encode_blit(struct virgl_context *ctx,
 
 int virgl_encoder_create_query(struct virgl_context *ctx,
                               uint32_t handle,
-                              uint query_type,
+                              uint32_t query_type,
                               struct virgl_resource *res,
                               uint32_t offset);
 
@@ -250,10 +317,14 @@ int virgl_encoder_end_query(struct virgl_context *ctx,
                            uint32_t handle);
 int virgl_encoder_get_query_result(struct virgl_context *ctx,
                                   uint32_t handle, boolean wait);
+int virgl_encoder_get_query_result_qbo(struct virgl_context *ctx,
+                                      uint32_t handle, uint32_t handle_qbo,
+                                      boolean wait_qbo, uint32_t result_type,
+                                      uint32_t type_qbo, uint32_t qbo_index);
 
 int virgl_encoder_render_condition(struct virgl_context *ctx,
                                   uint32_t handle, boolean condition,
-                                  uint mode);
+                                  uint32_t mode);
 
 int virgl_encoder_set_sub_ctx(struct virgl_context *ctx, uint32_t sub_ctx_id);
 int virgl_encoder_create_sub_ctx(struct virgl_context *ctx, uint32_t sub_ctx_id);
